@@ -3,16 +3,19 @@
 //
 // Creates the Kleene closure of an FST.
 
+#include <cstring>
+
 #include <memory>
+#include <string>
 
 #include <fst/script/closure.h>
+#include <fst/script/getters.h>
 
 DEFINE_bool(closure_plus, false,
-            "Do not add the empty path (T+ instead of T*)");
+            "Do not add the empty path (T+ instead of T*)?");
 
 int main(int argc, char **argv) {
   namespace s = fst::script;
-  using fst::script::FstClass;
   using fst::script::MutableFstClass;
 
   string usage = "Creates the Kleene closure of an FST.\n\n  Usage: ";
@@ -26,18 +29,13 @@ int main(int argc, char **argv) {
     return 1;
   }
 
-  string in_fname = (argc > 1 && strcmp(argv[1], "-") != 0) ? argv[1] : "";
-  string out_fname = argc > 2 ? argv[2] : "";
+  const string in_name = (argc > 1 && strcmp(argv[1], "-") != 0) ? argv[1] : "";
+  const string out_name = argc > 2 ? argv[2] : "";
 
-  std::unique_ptr<MutableFstClass> fst(MutableFstClass::Read(in_fname, true));
+  std::unique_ptr<MutableFstClass> fst(MutableFstClass::Read(in_name, true));
   if (!fst) return 1;
 
-  fst::ClosureType closure_type =
-      FLAGS_closure_plus ? fst::CLOSURE_PLUS : fst::CLOSURE_STAR;
+  s::Closure(fst.get(), s::GetClosureType(FLAGS_closure_plus));
 
-  s::Closure(fst.get(), closure_type);
-
-  fst->Write(out_fname);
-
-  return 0;
+  return !fst->Write(out_name);
 }
