@@ -3,9 +3,12 @@
 //
 // Concatenates two FSTs.
 
+#include <cstring>
+
 #include <memory>
 #include <string>
 
+#include <fst/log.h>
 #include <fst/script/concat.h>
 
 int main(int argc, char **argv) {
@@ -24,12 +27,12 @@ int main(int argc, char **argv) {
     return 1;
   }
 
-  string in1_name = strcmp(argv[1], "-") == 0 ? "" : argv[1];
-  string in2_name = strcmp(argv[2], "-") == 0 ? "" : argv[2];
-  string out_fname = argc > 3 ? argv[3] : "";
+  const string in1_name = strcmp(argv[1], "-") == 0 ? "" : argv[1];
+  const string in2_name = strcmp(argv[2], "-") == 0 ? "" : argv[2];
+  const string out_name = argc > 3 ? argv[3] : "";
 
   if (in1_name.empty() && in2_name.empty()) {
-    LOG(ERROR) << argv[0] << ": Can't take both inputs from standard input.";
+    LOG(ERROR) << argv[0] << ": Can't take both inputs from standard input";
     return 1;
   }
 
@@ -40,7 +43,6 @@ int main(int argc, char **argv) {
   if (!fst2) return 1;
 
   s::Concat(fst1.get(), *fst2);
-  fst1->Write(out_fname);
 
-  return 0;
+  return !fst1->Write(out_name);
 }
