@@ -4,7 +4,10 @@
 // Removes useless (inaccessible or non-coaccessible) states and arcs from an
 // FST.
 
+#include <cstring>
+
 #include <memory>
+#include <string>
 
 #include <fst/script/connect.h>
 
@@ -24,15 +27,13 @@ int main(int argc, char **argv) {
     return 1;
   }
 
-  string in_name = (argc > 1 && strcmp(argv[1], "-") != 0) ? argv[1] : "";
-  string out_name = argc > 2 ? argv[2] : "";
+  const string in_name = (argc > 1 && strcmp(argv[1], "-") != 0) ? argv[1] : "";
+  const string out_name = argc > 2 ? argv[2] : "";
 
   std::unique_ptr<MutableFstClass> fst(MutableFstClass::Read(in_name, true));
   if (!fst) return 1;
 
   s::Connect(fst.get());
 
-  fst->Write(out_name);
-
-  return 0;
+  return !fst->Write(out_name);
 }
