@@ -3,7 +3,10 @@
 //
 // Converts an FST to another type.
 
+#include <cstring>
+
 #include <memory>
+#include <string>
 
 #include <fst/script/convert.h>
 
@@ -24,8 +27,8 @@ int main(int argc, char **argv) {
     return 1;
   }
 
-  string in_name = (argc > 1 && strcmp(argv[1], "-") != 0) ? argv[1] : "";
-  string out_name = argc > 2 ? argv[2] : "";
+  const string in_name = (argc > 1 && strcmp(argv[1], "-") != 0) ? argv[1] : "";
+  const string out_name = argc > 2 ? argv[2] : "";
 
   std::unique_ptr<FstClass> ifst(FstClass::Read(in_name));
   if (!ifst) return 1;
@@ -33,10 +36,8 @@ int main(int argc, char **argv) {
   if (ifst->FstType() != FLAGS_fst_type) {
     std::unique_ptr<FstClass> ofst(s::Convert(*ifst, FLAGS_fst_type));
     if (!ofst) return 1;
-    ofst->Write(out_name);
+    return !ofst->Write(out_name);
   } else {
-    ifst->Write(out_name);
+    return !ifst->Write(out_name);
   }
-
-  return 0;
 }
