@@ -3,6 +3,12 @@
 //
 // Two FSTS are equal iff their exit status is zero.
 
+#include <cstring>
+
+#include <memory>
+#include <string>
+
+#include <fst/log.h>
 #include <fst/script/equal.h>
 
 DEFINE_double(delta, fst::kDelta, "Comparison/quantization delta");
@@ -22,11 +28,11 @@ int main(int argc, char **argv) {
     return 1;
   }
 
-  string in1_name = strcmp(argv[1], "-") == 0 ? "" : argv[1];
-  string in2_name = strcmp(argv[2], "-") == 0 ? "" : argv[2];
+  const string in1_name = strcmp(argv[1], "-") == 0 ? "" : argv[1];
+  const string in2_name = strcmp(argv[2], "-") == 0 ? "" : argv[2];
 
   if (in1_name.empty() && in2_name.empty()) {
-    LOG(ERROR) << argv[0] << ": Can't take both inputs from standard input.";
+    LOG(ERROR) << argv[0] << ": Can't take both inputs from standard input";
     return 1;
   }
 
