@@ -3,18 +3,23 @@
 //
 // Intersects two FSTs.
 
-#include <memory>
+#include <cstring>
 
-#include <fst/script/connect.h>
+#include <memory>
+#include <string>
+
+#include <fst/log.h>
+#include <fst/script/getters.h>
 #include <fst/script/intersect.h>
 
 DEFINE_string(compose_filter, "auto",
-              "Composition filter, one of: \"alt_sequence\", \"auto\", "
-              "\"match\", \"sequence\"");
+             "Composition filter, one of: \"alt_sequence\", \"auto\", "
+              "\"match\", \"null\", \"sequence\", \"trivial\"");
 DEFINE_bool(connect, true, "Trim output");
 
 int main(int argc, char **argv) {
   namespace s = fst::script;
+  using fst::ComposeFilter;
   using fst::script::FstClass;
   using fst::script::VectorFstClass;
 
@@ -30,12 +35,12 @@ int main(int argc, char **argv) {
     return 1;
   }
 
-  string in1_name = strcmp(argv[1], "-") == 0 ? "" : argv[1];
-  string in2_name = strcmp(argv[2], "-") == 0 ? "" : argv[2];
-  string out_name = argc > 3 ? argv[3] : "";
+  const string in1_name = strcmp(argv[1], "-") == 0 ? "" : argv[1];
+  const string in2_name = strcmp(argv[2], "-") == 0 ? "" : argv[2];
+  const string out_name = argc > 3 ? argv[3] : "";
 
   if (in1_name.empty() && in2_name.empty()) {
-    LOG(ERROR) << argv[0] << ": Can't take both inputs from standard input.";
+    LOG(ERROR) << argv[0] << ": Can't take both inputs from standard input";
     return 1;
   }
 
@@ -46,27 +51,16 @@ int main(int argc, char **argv) {
 
   VectorFstClass ofst(ifst1->ArcType());
 
-  fst::ComposeFilter compose_filter;
-
-  if (FLAGS_compose_filter == "alt_sequence") {
-    compose_filter = fst::ALT_SEQUENCE_FILTER;
-  } else if (FLAGS_compose_filter == "auto") {
-    compose_filter = fst::AUTO_FILTER;
-  } else if (FLAGS_compose_filter == "match") {
-    compose_filter = fst::MATCH_FILTER;
-  } else if (FLAGS_compose_filter == "sequence") {
-    compose_filter = fst::SEQUENCE_FILTER;
-  } else {
-    LOG(ERROR) << argv[0]
-               << "Unknown compose filter type: " << FLAGS_compose_filter;
+  ComposeFilter compose_filter;
+  if (!s::GetComposeFilter(FLAGS_compose_filter, &compose_filter)) {
+    LOG(ERROR) << argv[0] << ": Unknown or unsupported compose filter type: "
+               << FLAGS_compose_filter;
     return 1;
   }
 
-  fst::IntersectOptions opts(FLAGS_connect, compose_filter);
+  const fst::IntersectOptions opts(FLAGS_connect, compose_filter);
 
   s::Intersect(*ifst1, *ifst2, &ofst, opts);
 
-  ofst.Write(out_name);
-
-  return 0;
+  return !ofst.Write(out_name);
 }
