@@ -3,13 +3,15 @@
 //
 // Inverts a transduction.
 
+#include <cstring>
+
 #include <memory>
+#include <string>
 
 #include <fst/script/invert.h>
 
 int main(int argc, char **argv) {
   namespace s = fst::script;
-  using fst::script::FstClass;
   using fst::script::MutableFstClass;
 
   string usage = "Inverts a transduction.\n\n  Usage: ";
@@ -23,14 +25,13 @@ int main(int argc, char **argv) {
     return 1;
   }
 
-  string in_name = (argc > 1 && strcmp(argv[1], "-") != 0) ? argv[1] : "";
-  string out_name = argc > 2 ? argv[2] : "";
+  const string in_name = (argc > 1 && strcmp(argv[1], "-") != 0) ? argv[1] : "";
+  const string out_name = argc > 2 ? argv[2] : "";
 
   std::unique_ptr<MutableFstClass> fst(MutableFstClass::Read(in_name, true));
   if (!fst) return 1;
 
   s::Invert(fst.get());
-  fst->Write(out_name);
 
-  return 0;
+  return !fst->Write(out_name);
 }
