@@ -3,15 +3,18 @@
 //
 // Projects a transduction onto its input or output language.
 
-#include <memory>
+#include <cstring>
 
+#include <memory>
+#include <string>
+
+#include <fst/script/getters.h>
 #include <fst/script/project.h>
 
 DEFINE_bool(project_output, false, "Project on output (vs. input)");
 
 int main(int argc, char **argv) {
   namespace s = fst::script;
-  using fst::script::FstClass;
   using fst::script::MutableFstClass;
 
   string usage =
@@ -27,18 +30,13 @@ int main(int argc, char **argv) {
     return 1;
   }
 
-  string in_name = (argc > 1 && strcmp(argv[1], "-") != 0) ? argv[1] : "";
-  string out_name = argc > 2 ? argv[2] : "";
+  const string in_name = (argc > 1 && strcmp(argv[1], "-") != 0) ? argv[1] : "";
+  const string out_name = argc > 2 ? argv[2] : "";
 
   std::unique_ptr<MutableFstClass> fst(MutableFstClass::Read(in_name, true));
   if (!fst) return 1;
 
-  fst::ProjectType project_type =
-      FLAGS_project_output ? fst::PROJECT_OUTPUT : fst::PROJECT_INPUT;
+  s::Project(fst.get(), s::GetProjectType(FLAGS_project_output));
 
-  s::Project(fst.get(), project_type);
-
-  fst->Write(out_name);
-
-  return 0;
+  return !fst->Write(out_name);
 }
