@@ -3,8 +3,13 @@
 //
 // Reweights an FST.
 
-#include <memory>
+#include <cstring>
 
+#include <memory>
+#include <string>
+#include <vector>
+
+#include <fst/script/getters.h>
 #include <fst/script/reweight.h>
 #include <fst/script/text-io.h>
 
@@ -12,8 +17,8 @@ DEFINE_bool(to_final, false, "Push/reweight to final (vs. to initial) states");
 
 int main(int argc, char **argv) {
   namespace s = fst::script;
-  using fst::script::FstClass;
   using fst::script::MutableFstClass;
+  using fst::script::WeightClass;
 
   string usage = "Reweights an FST.\n\n  Usage: ";
   usage += argv[0];
@@ -26,24 +31,19 @@ int main(int argc, char **argv) {
     return 1;
   }
 
-  string in_fname = argv[1];
-  string potentials_fname = argv[2];
-  string out_fname = argc > 3 ? argv[3] : "";
+  const string in_name = argv[1];
+  const string potentials_name = argv[2];
+  const string out_name = argc > 3 ? argv[3] : "";
 
-  std::unique_ptr<MutableFstClass> fst(MutableFstClass::Read(in_fname, true));
+  std::unique_ptr<MutableFstClass> fst(MutableFstClass::Read(in_name, true));
   if (!fst) return 1;
 
-  std::vector<s::WeightClass> potential;
-  if (!s::ReadPotentials(fst->WeightType(), potentials_fname, &potential))
+  std::vector<WeightClass> potential;
+  if (!s::ReadPotentials(fst->WeightType(), potentials_name, &potential)) {
     return 1;
+  }
 
-  fst::ReweightType reweight_type = FLAGS_to_final
-                                            ? fst::REWEIGHT_TO_FINAL
-                                            : fst::REWEIGHT_TO_INITIAL;
+  s::Reweight(fst.get(), potential, s::GetReweightType(FLAGS_to_final));
 
-  s::Reweight(fst.get(), potential, reweight_type);
-
-  fst->Write(out_fname);
-
-  return 0;
+  return !fst->Write(out_name);
 }
