@@ -3,8 +3,13 @@
 //
 // Removes epsilons from an FST.
 
-#include <memory>
+#include <cstring>
 
+#include <memory>
+#include <string>
+
+#include <fst/log.h>
+#include <fst/script/getters.h>
 #include <fst/script/rmepsilon.h>
 
 DEFINE_bool(connect, true, "Trim output");
@@ -19,7 +24,6 @@ DEFINE_string(queue_type, "auto",
 int main(int argc, char **argv) {
   namespace s = fst::script;
   using fst::script::FstClass;
-  using fst::script::MutableFstClass;
   using fst::script::VectorFstClass;
   using fst::script::WeightClass;
 
@@ -34,43 +38,29 @@ int main(int argc, char **argv) {
     return 1;
   }
 
-  string in_fname = (argc > 1 && strcmp(argv[1], "-") != 0) ? argv[1] : "";
-  string out_fname = argc > 2 ? argv[2] : "";
+  const string in_name = (argc > 1 && strcmp(argv[1], "-") != 0) ? argv[1] : "";
+  const string out_name = argc > 2 ? argv[2] : "";
 
-  std::unique_ptr<FstClass> ifst(FstClass::Read(in_fname));
+  std::unique_ptr<FstClass> ifst(FstClass::Read(in_name));
   if (!ifst) return 1;
 
-  WeightClass weight_threshold =
+  const auto weight_threshold =
       FLAGS_weight.empty() ? WeightClass::Zero(ifst->WeightType())
                            : WeightClass(ifst->WeightType(), FLAGS_weight);
 
-  fst::QueueType qt;
-
-  if (FLAGS_queue_type == "auto") {
-    qt = fst::AUTO_QUEUE;
-  } else if (FLAGS_queue_type == "fifo") {
-    qt = fst::FIFO_QUEUE;
-  } else if (FLAGS_queue_type == "lifo") {
-    qt = fst::LIFO_QUEUE;
-  } else if (FLAGS_queue_type == "shortest") {
-    qt = fst::SHORTEST_FIRST_QUEUE;
-  } else if (FLAGS_queue_type == "state") {
-    qt = fst::STATE_ORDER_QUEUE;
-  } else if (FLAGS_queue_type == "top") {
-    qt = fst::TOP_ORDER_QUEUE;
-  } else {
+  fst::QueueType queue_type;
+  if (!s::GetQueueType(FLAGS_queue_type, &queue_type)) {
     LOG(ERROR) << argv[0]
                << ": Unknown or unsupported queue type: " << FLAGS_queue_type;
     return 1;
   }
 
-  s::RmEpsilonOptions opts(qt, FLAGS_delta, FLAGS_connect, weight_threshold,
-                           FLAGS_nstate);
+  const s::RmEpsilonOptions opts(queue_type, FLAGS_delta, FLAGS_connect,
+                                 weight_threshold, FLAGS_nstate);
 
   VectorFstClass ofst(ifst->ArcType());
+
   s::RmEpsilon(*ifst, &ofst, FLAGS_reverse, opts);
 
-  ofst.Write(out_fname);
-
-  return 0;
+  return !ofst.Write(out_name);
 }
