@@ -3,9 +3,14 @@
 //
 // Find shortest distances in an FST.
 
+#include <cstring>
+
 #include <memory>
+#include <string>
 #include <vector>
 
+#include <fst/log.h>
+#include <fst/script/getters.h>
 #include <fst/script/shortest-distance.h>
 #include <fst/script/text-io.h>
 
@@ -20,6 +25,8 @@ int main(int argc, char **argv) {
   namespace s = fst::script;
   using fst::script::FstClass;
   using fst::script::WeightClass;
+  using fst::QueueType;
+  using fst::AUTO_QUEUE;
 
   string usage = "Finds shortest distance(s) in an FST.\n\n  Usage: ";
   usage += argv[0];
@@ -32,48 +39,33 @@ int main(int argc, char **argv) {
     return 1;
   }
 
-  string in_fname = (argc > 1 && (strcmp(argv[1], "-") != 0)) ? argv[1] : "";
-  string out_fname = argc > 2 ? argv[2] : "";
+  string in_name = (argc > 1 && (strcmp(argv[1], "-") != 0)) ? argv[1] : "";
+  string out_name = argc > 2 ? argv[2] : "";
 
-  std::unique_ptr<FstClass> ifst(FstClass::Read(in_fname));
+  std::unique_ptr<FstClass> ifst(FstClass::Read(in_name));
   if (!ifst) return 1;
 
   std::vector<WeightClass> distance;
 
-  fst::QueueType qt;
-
-  if (FLAGS_queue_type == "auto") {
-    qt = fst::AUTO_QUEUE;
-  } else if (FLAGS_queue_type == "fifo") {
-    qt = fst::FIFO_QUEUE;
-  } else if (FLAGS_queue_type == "lifo") {
-    qt = fst::LIFO_QUEUE;
-  } else if (FLAGS_queue_type == "shortest") {
-    qt = fst::SHORTEST_FIRST_QUEUE;
-  } else if (FLAGS_queue_type == "state") {
-    qt = fst::STATE_ORDER_QUEUE;
-  } else if (FLAGS_queue_type == "top") {
-    qt = fst::TOP_ORDER_QUEUE;
-  } else {
+  QueueType queue_type;
+  if (!s::GetQueueType(FLAGS_queue_type, &queue_type)) {
     LOG(ERROR) << argv[0]
                << ": Unknown or unsupported queue type: " << FLAGS_queue_type;
     return 1;
   }
 
-  if (FLAGS_reverse && qt != fst::AUTO_QUEUE) {
-    LOG(ERROR) << argv[0] << ": Non-default queue with reverse not supported.";
+  if (FLAGS_reverse && queue_type != AUTO_QUEUE) {
+    LOG(ERROR) << argv[0] << ": Can't use non-default queue with reverse";
     return 1;
   }
 
   if (FLAGS_reverse) {
     s::ShortestDistance(*ifst, &distance, FLAGS_reverse, FLAGS_delta);
   } else {
-    s::ShortestDistanceOptions opts(qt, s::ANY_ARC_FILTER, FLAGS_nstate,
-                                    FLAGS_delta);
+    const s::ShortestDistanceOptions opts(queue_type, s::ANY_ARC_FILTER,
+                                          FLAGS_nstate, FLAGS_delta);
     s::ShortestDistance(*ifst, &distance, opts);
   }
 
-  s::WritePotentials(out_fname, distance);
-
-  return 0;
+  return !s::WritePotentials(out_name, distance);
 }
