@@ -3,8 +3,14 @@
 //
 // Find shortest path(s) in an FST.
 
-#include <memory>
+#include <cstring>
 
+#include <memory>
+#include <string>
+#include <vector>
+
+#include <fst/log.h>
+#include <fst/script/getters.h>
 #include <fst/script/shortest-path.h>
 
 DEFINE_double(delta, fst::kDelta, "Comparison/quantization delta");
@@ -19,7 +25,6 @@ DEFINE_string(queue_type, "auto",
 int main(int argc, char **argv) {
   namespace s = fst::script;
   using fst::script::FstClass;
-  using fst::script::MutableFstClass;
   using fst::script::WeightClass;
   using fst::script::VectorFstClass;
 
@@ -34,37 +39,26 @@ int main(int argc, char **argv) {
     return 1;
   }
 
-  string in_fname = (argc > 1 && (strcmp(argv[1], "-") != 0)) ? argv[1] : "";
-  string out_fname = argc > 2 ? argv[2] : "";
+  const string in_name =
+      (argc > 1 && (strcmp(argv[1], "-") != 0)) ? argv[1] : "";
+  const string out_name = argc > 2 ? argv[2] : "";
 
-  std::unique_ptr<FstClass> ifst(FstClass::Read(in_fname));
+  std::unique_ptr<FstClass> ifst(FstClass::Read(in_name));
   if (!ifst) return 1;
 
-  WeightClass weight_threshold =
+  const auto weight_threshold =
       FLAGS_weight.empty() ? WeightClass::Zero(ifst->WeightType())
                            : WeightClass(ifst->WeightType(), FLAGS_weight);
 
   VectorFstClass ofst(ifst->ArcType());
 
-  fst::QueueType qt;
-  if (FLAGS_queue_type == "auto") {
-    qt = fst::AUTO_QUEUE;
-  } else if (FLAGS_queue_type == "fifo") {
-    qt = fst::FIFO_QUEUE;
-  } else if (FLAGS_queue_type == "lifo") {
-    qt = fst::LIFO_QUEUE;
-  } else if (FLAGS_queue_type == "shortest") {
-    qt = fst::SHORTEST_FIRST_QUEUE;
-  } else if (FLAGS_queue_type == "state") {
-    qt = fst::STATE_ORDER_QUEUE;
-  } else if (FLAGS_queue_type == "top") {
-    qt = fst::TOP_ORDER_QUEUE;
-  } else {
+  fst::QueueType queue_type;
+  if (!s::GetQueueType(FLAGS_queue_type, &queue_type)) {
     LOG(ERROR) << "Unknown or unsupported queue type: " << FLAGS_queue_type;
     return 1;
   }
 
-  s::ShortestPathOptions opts(qt, FLAGS_nshortest, FLAGS_unique, false,
+  s::ShortestPathOptions opts(queue_type, FLAGS_nshortest, FLAGS_unique, false,
                               FLAGS_delta, false, weight_threshold,
                               FLAGS_nstate);
 
@@ -72,7 +66,5 @@ int main(int argc, char **argv) {
 
   s::ShortestPath(*ifst, &ofst, &distance, opts);
 
-  ofst.Write(out_fname);
-
-  return 0;
+  return !ofst.Write(out_name);
 }
