@@ -3,7 +3,10 @@
 //
 // Synchronizes an FST.
 
+#include <cstring>
+
 #include <memory>
+#include <string>
 
 #include <fst/script/synchronize.h>
 
@@ -23,8 +26,8 @@ int main(int argc, char **argv) {
     return 1;
   }
 
-  string in_name = (argc > 1 && strcmp(argv[1], "-") != 0) ? argv[1] : "";
-  string out_name = argc > 2 ? argv[2] : "";
+  const string in_name = (argc > 1 && strcmp(argv[1], "-") != 0) ? argv[1] : "";
+  const string out_name = argc > 2 ? argv[2] : "";
 
   std::unique_ptr<FstClass> ifst(FstClass::Read(in_name));
   if (!ifst) return 1;
@@ -33,7 +36,5 @@ int main(int argc, char **argv) {
 
   s::Synchronize(*ifst, &ofst);
 
-  ofst.Write(out_name);
-
-  return 0;
+  return !ofst.Write(out_name);
 }
