@@ -3,13 +3,16 @@
 //
 // Topologically sorts an FST.
 
-#include <memory>
+#include <cstring>
 
+#include <memory>
+#include <string>
+
+#include <fst/log.h>
 #include <fst/script/topsort.h>
 
 int main(int argc, char **argv) {
   namespace s = fst::script;
-  using fst::script::FstClass;
   using fst::script::MutableFstClass;
 
   string usage = "Topologically sorts an FST.\n\n  Usage: ";
@@ -23,16 +26,15 @@ int main(int argc, char **argv) {
     return 1;
   }
 
-  string in_fname = argc > 1 && strcmp(argv[1], "-") != 0 ? argv[1] : "";
-  string out_fname = argc > 2 ? argv[2] : "";
+  const string in_name = argc > 1 && strcmp(argv[1], "-") != 0 ? argv[1] : "";
+  const string out_name = argc > 2 ? argv[2] : "";
 
-  std::unique_ptr<MutableFstClass> fst(MutableFstClass::Read(in_fname, true));
+  std::unique_ptr<MutableFstClass> fst(MutableFstClass::Read(in_name, true));
   if (!fst) return 1;
 
   bool acyclic = TopSort(fst.get());
+
   if (!acyclic) LOG(WARNING) << argv[0] << ": Input FST is cyclic";
 
-  fst->Write(out_fname);
-
-  return 0;
+  return !fst->Write(out_name);
 }
