@@ -3,6 +3,12 @@
 //
 // Creates the union of two FSTs.
 
+#include <cstring>
+
+#include <memory>
+#include <string>
+
+#include <fst/log.h>
 #include <fst/script/union.h>
 
 int main(int argc, char **argv) {
@@ -21,24 +27,22 @@ int main(int argc, char **argv) {
     return 1;
   }
 
-  string in1_name = strcmp(argv[1], "-") != 0 ? argv[1] : "";
-  string in2_name = strcmp(argv[2], "-") != 0 ? argv[2] : "";
-  string out_name = argc > 3 ? argv[3] : "";
+  const string in1_name = strcmp(argv[1], "-") != 0 ? argv[1] : "";
+  const string in2_name = strcmp(argv[2], "-") != 0 ? argv[2] : "";
+  const string out_name = argc > 3 ? argv[3] : "";
 
   if (in1_name == "" && in2_name == "") {
-    LOG(ERROR) << argv[0] << ": Can't use standard i/o for both inputs.";
+    LOG(ERROR) << argv[0] << ": Can't take both inputs from standard input";
     return 1;
   }
 
-  MutableFstClass *fst1 = MutableFstClass::Read(in1_name, true);
+  std::unique_ptr<MutableFstClass> fst1(MutableFstClass::Read(in1_name, true));
   if (!fst1) return 1;
 
-  FstClass *fst2 = FstClass::Read(in2_name);
+  std::unique_ptr<FstClass> fst2(FstClass::Read(in2_name));
   if (!fst2) return 1;
 
-  s::Union(fst1, *fst2);
+  s::Union(fst1.get(), *fst2);
 
-  fst1->Write(out_name);
-
-  return 0;
+  return !fst1->Write(out_name);
 }
