@@ -4,10 +4,7 @@
 #include <fst/fst.h>
 #include <fst/compact-fst.h>
 
-using fst::FstRegisterer;
-using fst::CompactUnweightedAcceptorFst;
-using fst::LogArc;
-using fst::StdArc;
+namespace fst {
 
 static FstRegisterer<
     CompactUnweightedAcceptorFst<StdArc, uint8>>
@@ -15,3 +12,5 @@ static FstRegisterer<
 static FstRegisterer<
     CompactUnweightedAcceptorFst<LogArc, uint8>>
     CompactUnweightedAcceptorFst_LogArc_uint8_registerer;
+
+}  // namespace fst
