@@ -3,7 +3,11 @@
 //
 // Outputs as strings the string FSTs in a finite-state archive.
 
+#include <string>
+#include <vector>
+
 #include <fst/extensions/far/farscript.h>
+#include <fst/extensions/far/getters.h>
 
 DEFINE_string(filename_prefix, "", "Prefix to append to filenames");
 DEFINE_string(filename_suffix, "", "Suffix to append to filenames");
@@ -28,27 +32,38 @@ DEFINE_bool(initial_symbols, true,
 int main(int argc, char **argv) {
   namespace s = fst::script;
 
-  string usage = "Print as string the string FSTs in an archive.\n\n Usage:";
+  string usage = "Print as string the string FSTs in an archive.\n\n  Usage:";
   usage += argv[0];
   usage += " [in1.far in2.far ...]\n";
 
   std::set_new_handler(FailedNewHandler);
   SET_FLAGS(usage.c_str(), &argc, &argv, true);
-  fst::ExpandArgs(argc, argv, &argc, &argv);
+  s::ExpandArgs(argc, argv, &argc, &argv);
 
-  std::vector<string> ifilenames;
-  for (int i = 1; i < argc; ++i)
-    ifilenames.push_back(strcmp(argv[i], "") != 0 ? argv[i] : "");
-  if (ifilenames.empty()) ifilenames.push_back("");
+  std::vector<string> in_fnames;
+  for (int i = 1; i < argc; ++i) in_fnames.push_back(argv[i]);
+  if (in_fnames.empty()) in_fnames.push_back("");
 
-  string arc_type = fst::LoadArcTypeFromFar(ifilenames[0]);
+  const auto arc_type = s::LoadArcTypeFromFar(in_fnames[0]);
+  if (arc_type.empty()) return 1;
 
-  s::FarPrintStrings(
-      ifilenames, arc_type, fst::StringToFarEntryType(FLAGS_entry_type),
-      fst::StringToFarTokenType(FLAGS_token_type), FLAGS_begin_key,
-      FLAGS_end_key, FLAGS_print_key, FLAGS_print_weight, FLAGS_symbols,
-      FLAGS_initial_symbols, FLAGS_generate_filenames, FLAGS_filename_prefix,
-      FLAGS_filename_suffix);
+  fst::FarEntryType entry_type;
+  if (!s::GetFarEntryType(FLAGS_entry_type, &entry_type)) {
+    LOG(ERROR) << "Unknown or unsupported FAR entry type: " << FLAGS_entry_type;
+    return 1;
+  }
+
+  fst::FarTokenType token_type;
+  if (!s::GetFarTokenType(FLAGS_token_type, &token_type)) {
+    LOG(ERROR) << "Unknown or unsupported FAR token type: " << FLAGS_token_type;
+    return 1;
+  }
+
+  s::FarPrintStrings(in_fnames, arc_type, entry_type, token_type,
+                     FLAGS_begin_key, FLAGS_end_key, FLAGS_print_key,
+                     FLAGS_print_weight, FLAGS_symbols, FLAGS_initial_symbols,
+                     FLAGS_generate_filenames, FLAGS_filename_prefix,
+                     FLAGS_filename_suffix);
 
   return 0;
 }
