@@ -9,10 +9,8 @@
 namespace fst {
 
 bool IsSTList(const string &filename) {
-  std::ifstream strm(filename.c_str(),
-                          std::ios_base::in | std::ios_base::binary);
+  std::ifstream strm(filename, std::ios_base::in | std::ios_base::binary);
   if (!strm) return false;
-
   int32 magic_number = 0;
   ReadType(strm, &magic_number);
   return magic_number == kSTListMagicNumber;
