@@ -14,72 +14,69 @@
 namespace fst {
 
 // True for all arcs.
-template <class A>
+template <class Arc>
 class AnyArcFilter {
  public:
-  bool operator()(const A &arc) const { return true; }
+  bool operator()(const Arc &arc) const { return true; }
 };
 
 // True for (input/output) epsilon arcs.
-template <class A>
+template <class Arc>
 class EpsilonArcFilter {
  public:
-  bool operator()(const A &arc) const {
+  bool operator()(const Arc &arc) const {
     return arc.ilabel == 0 && arc.olabel == 0;
   }
 };
 
 // True for input epsilon arcs.
-template <class A>
+template <class Arc>
 class InputEpsilonArcFilter {
  public:
-  bool operator()(const A &arc) const { return arc.ilabel == 0; }
+  bool operator()(const Arc &arc) const { return arc.ilabel == 0; }
 };
 
 // True for output epsilon arcs.
-template <class A>
+template <class Arc>
 class OutputEpsilonArcFilter {
  public:
-  bool operator()(const A &arc) const { return arc.olabel == 0; }
+  bool operator()(const Arc &arc) const { return arc.olabel == 0; }
 };
 
 // True if specified label matches (doesn't match) when keep_match is
 // true (false).
-template <class A>
+template <class Arc>
 class LabelArcFilter {
  public:
-  typedef typename A::Label Label;
+  using Label = typename Arc::Label;
 
   explicit LabelArcFilter(Label label, bool match_input = true,
                           bool keep_match = true)
       : label_(label), match_input_(match_input), keep_match_(keep_match) {}
 
-  bool operator()(const A &arc) const {
-    Label label = match_input_ ? arc.ilabel : arc.olabel;
-    bool match = label == label_;
+  bool operator()(const Arc &arc) const {
+    const bool match = (match_input_ ? arc.ilabel : arc.olabel) == label_;
     return keep_match_ ? match : !match;
   }
 
  private:
-  Label label_;
-  bool match_input_;
-  bool keep_match_;
+  const Label label_;
+  const bool match_input_;
+  const bool keep_match_;
 };
 
-
-// True if specified labels match (don't match) when keep_match is
-// true (false).
-template <class A>
+// True if specified labels match (don't match) when keep_match is true (false).
+template <class Arc>
 class MultiLabelArcFilter {
  public:
-  typedef typename A::Label Label;
+  using Label = typename Arc::Label;
 
-  MultiLabelArcFilter(bool match_input = true, bool keep_match = true)
+  explicit MultiLabelArcFilter(bool match_input = true, bool keep_match = true)
       : match_input_(match_input), keep_match_(keep_match) {}
 
-  bool operator()(const A &arc) const {
-    Label label = match_input_ ? arc.ilabel : arc.olabel;
-    bool match = labels_.Find(label) != labels_.End();
+  bool operator()(const Arc &arc) const {
+    const Label label = match_input_ ? arc.ilabel : arc.olabel;
+    const bool match = labels_.Find(label) != labels_.End();
     return keep_match_ ? match : !match;
   }
 
@@ -87,8 +84,8 @@ class MultiLabelArcFilter {
 
  private:
   CompactSet<Label, kNoLabel> labels_;
-  bool match_input_;
-  bool keep_match_;
+  const bool match_input_;
+  const bool keep_match_;
 };
 
 }  // namespace fst
