@@ -14,13 +14,14 @@
 #include <unordered_set>
 #include <vector>
 
+#include <fst/log.h>
 #include <fst/memory.h>
 
 namespace fst {
 
-// BI TABLES - these determine a bijective mapping between an
-// arbitrary entry of type T and an signed integral ID of type I. The IDs are
-// allocated starting from 0 in order.
+// Bitables model bijective mappings between entries of an arbitrary type T and
+// an signed integral ID of type I. The IDs are allocated starting from 0 in
+// order.
 //
 // template <class I, class T>
 // class BiTable {
@@ -29,23 +30,24 @@ namespace fst {
 //   // Required constructors.
 //   BiTable();
 //
-//   // Lookup integer ID from entry. If it doesn't exist and 'insert'
-//   / is true, then add it. Otherwise return -1.
+//   // Looks up integer ID from entry. If it doesn't exist and insert
+//   / is true, adds it; otherwise, returns -1.
 //   I FindId(const T &entry, bool insert = true);
-//   // Lookup entry from integer ID.
+//
+//   // Looks up entry from integer ID.
 //   const T &FindEntry(I) const;
-//   // # of stored entries.
+//
+//   // Returns number of stored entries.
 //   I Size() const;
 // };
 
-// An implementation using a hash map for the entry to ID mapping.
-// H is the hash function and E is the equality function.
-// If passed to the constructor, ownership is given to this class.
-
+// An implementation using a hash map for the entry to ID mapping. H is the
+// hash function and E is the equality function. If passed to the constructor,
+// ownership is given to this class.
 template <class I, class T, class H, class E = std::equal_to<T>>
 class HashBiTable {
  public:
-  // Reserves space for 'table_size' elements. If passing H and E to the
+  // Reserves space for table_size elements. If passing H and E to the
   // constructor, this class owns them.
   explicit HashBiTable(size_t table_size = 0, H *h = nullptr, E *e = nullptr) :
       hash_func_(h ? h : new H()), hash_equal_(e ? e : new E()),
@@ -66,18 +68,18 @@ class HashBiTable {
       return it == entry2id_.end() ? -1 : it->second - 1;
     }
     I &id_ref = entry2id_[entry];
-    if (id_ref == 0) {  // T not found  store and assign it a new ID
+    if (id_ref == 0) {  // T not found; stores and assigns a new ID.
       id2entry_.push_back(entry);
       id_ref = id2entry_.size();
     }
-    return id_ref - 1;  // NB: id_ref = ID + 1
+    return id_ref - 1;  // NB: id_ref = ID + 1.
   }
 
   const T &FindEntry(I s) const { return id2entry_[s]; }
 
   I Size() const { return id2entry_.size(); }
 
-  // TODO(riley): Add fancy clear-to-size as in CompactHashBiTable.
+  // TODO(riley): Add fancy clear-to-size, as in CompactHashBiTable.
   void Clear() {
     entry2id_.clear();
     id2entry_.clear();
@@ -88,29 +90,26 @@ class HashBiTable {
   std::unique_ptr<E> hash_equal_;
   std::unordered_map<T, I, H, E> entry2id_;
   std::vector<T> id2entry_;
-
-  void operator=(const HashBiTable<I, T, H, E> &table);  // disallow
 };
 
 // Enables alternative hash set representations below.
-typedef enum { HS_STL = 0, HS_DENSE = 1, HS_SPARSE = 2 } HSType;
+enum HSType { HS_STL = 0, HS_DENSE = 1, HS_SPARSE = 2 };
 
-// Default hash set is STL hash_set
-template <class K, class H, class E, HSType>
+// Default hash set is STL hash_set.
+template <class K, class H, class E, HSType HS>
 struct HashSet : public std::unordered_set<K, H, E, PoolAllocator<K>> {
-  HashSet(size_t n = 0, const H &h = H(), const E &e = E())
+  explicit HashSet(size_t n = 0, const H &h = H(), const E &e = E())
       : std::unordered_set<K, H, E, PoolAllocator<K>>(n, h, e) {}
 
   void rehash(size_t n) {}
 };
 
-// An implementation using a hash set for the entry to ID mapping.
-// The hash set holds 'keys' which are either the ID or kCurrentKey.
-// These keys can be mapped to entrys either by looking up in the
-// entry vector or, if kCurrentKey, in current_entry_ member. The hash
-// and key equality functions map to entries first. H
-// is the hash function and E is the equality function. If passed to
-// the constructor, ownership is given to this class.
+// An implementation using a hash set for the entry to ID mapping. The hash set
+// holds keys which are either the ID or kCurrentKey. These keys can be mapped
+// to entries either by looking up in the entry vector or, if kCurrentKey, in
+// current_entry_. The hash and key equality functions map to entries first. H
+// is the hash function and E is the equality function. If passed to the
+// constructor, ownership is given to this class.
 template <class I, class T, class H, class E = std::equal_to<T>,
           HSType HS = HS_DENSE>
 class CompactHashBiTable {
@@ -118,7 +117,7 @@ class CompactHashBiTable {
   friend class HashFunc;
   friend class HashEqual;
 
-  // Reserves space for 'table_size' elements. If passing H and E to the
+  // Reserves space for table_size elements. If passing H and E to the
   // constructor, this class owns them.
   explicit CompactHashBiTable(size_t table_size = 0, H *h = nullptr,
                               E *e = nullptr) :
@@ -139,31 +138,30 @@ class CompactHashBiTable {
 
   I FindId(const T &entry, bool insert = true) {
     current_entry_ = &entry;
-    const auto it = keys_.find(kCurrentKey);
-    if (it == keys_.end()) {  // T not found
-      if (insert) {           // store and assign it a new ID
-        I key = id2entry_.size();
-        id2entry_.push_back(entry);
-        keys_.insert(key);
-        return key;
-      } else {
-        return -1;
-      }
-    } else {
-      return *it;
+    if (insert) {
+      auto result = keys_.insert(kCurrentKey);
+      if (!result.second) return *result.first;  // Already exists.
+      // Overwrites kCurrentKey with a new key value; this is safe because it
+      // doesn't affect hashing or equality testing.
+      I key = id2entry_.size();
+      const_cast<I &>(*result.first) = key;
+      id2entry_.push_back(entry);
+      return key;
     }
+    const auto it = keys_.find(kCurrentKey);
+    return it == keys_.end() ? -1 : *it;
   }
 
   const T &FindEntry(I s) const { return id2entry_[s]; }
 
   I Size() const { return id2entry_.size(); }
 
-  // Clear content. With argument, erases last n IDs.
+  // Clears content; with argument, erases last n IDs.
   void Clear(ssize_t n = -1) {
-    if (n < 0 || n >= id2entry_.size()) {  // Clear completely.
+    if (n < 0 || n >= id2entry_.size()) {  // Clears completely.
       keys_.clear();
       id2entry_.clear();
-    } else if (n == id2entry_.size() - 1) {  // Leave only key 0
+    } else if (n == id2entry_.size() - 1) {  // Leaves only key 0.
       const T entry = FindEntry(0);
       keys_.clear();
       id2entry_.clear();
@@ -179,13 +177,13 @@ class CompactHashBiTable {
   }
 
  private:
-  static const I kCurrentKey;  // -1
-  static const I kEmptyKey;    // -2
-  static const I kDeletedKey;  // -3
+  static constexpr I kCurrentKey = -1;
+  static constexpr I kEmptyKey = -2;
+  static constexpr I kDeletedKey = -3;
 
   class HashFunc {
    public:
-    HashFunc(const CompactHashBiTable &ht) : ht_(&ht) {}
+    explicit HashFunc(const CompactHashBiTable &ht) : ht_(&ht) {}
 
     size_t operator()(I k) const {
       if (k >= kCurrentKey) {
@@ -201,13 +199,15 @@ class CompactHashBiTable {
 
   class HashEqual {
    public:
-    HashEqual(const CompactHashBiTable &ht) : ht_(&ht) {}
+    explicit HashEqual(const CompactHashBiTable &ht) : ht_(&ht) {}
 
     bool operator()(I k1, I k2) const {
-      if (k1 >= kCurrentKey && k2 >= kCurrentKey) {
+      if (k1 == k2) {
+        return true;
+      } else if (k1 >= kCurrentKey && k2 >= kCurrentKey) {
         return (*ht_->hash_equal_)(ht_->Key2Entry(k1), ht_->Key2Entry(k2));
       } else {
-        return k1 == k2;
+        return false;
       }
     }
 
@@ -215,13 +215,14 @@ class CompactHashBiTable {
     const CompactHashBiTable *ht_;
   };
 
-  typedef HashSet<I, HashFunc, HashEqual, HS> KeyHashSet;
+  using KeyHashSet = HashSet<I, HashFunc, HashEqual, HS>;
 
   const T &Key2Entry(I k) const {
-    if (k == kCurrentKey)
+    if (k == kCurrentKey) {
       return *current_entry_;
-    else
+    } else {
       return id2entry_[k];
+    }
   }
 
   std::unique_ptr<H> hash_func_;
@@ -231,29 +232,26 @@ class CompactHashBiTable {
   KeyHashSet keys_;
   std::vector<T> id2entry_;
   const T *current_entry_;
-
-  void operator=(const CompactHashBiTable<I, T, H, E, HS> &table);  // disallow
 };
 
 template <class I, class T, class H, class E, HSType HS>
-const I CompactHashBiTable<I, T, H, E, HS>::kCurrentKey = -1;
+constexpr I CompactHashBiTable<I, T, H, E, HS>::kCurrentKey;
 
 template <class I, class T, class H, class E, HSType HS>
-const I CompactHashBiTable<I, T, H, E, HS>::kEmptyKey = -2;
+constexpr I CompactHashBiTable<I, T, H, E, HS>::kEmptyKey;
 
 template <class I, class T, class H, class E, HSType HS>
-const I CompactHashBiTable<I, T, H, E, HS>::kDeletedKey = -3;
+constexpr I CompactHashBiTable<I, T, H, E, HS>::kDeletedKey;
 
-// An implementation using a vector for the entry to ID mapping.
-// It is passed a function object FP that should fingerprint entries
-// uniquely to an integer that can used as a vector index. Normally,
-// VectorBiTable constructs the FP object.  The user can instead
-// pass in this object; in that case, VectorBiTable takes its
-// ownership.
+// An implementation using a vector for the entry to ID mapping. It is passed a
+// function object FP that should fingerprint entries uniquely to an integer
+// that can used as a vector index. Normally, VectorBiTable constructs the FP
+// object. The user can instead pass in this object; in that case, VectorBiTable
+// takes its ownership.
 template <class I, class T, class FP>
 class VectorBiTable {
  public:
-  // Reserves space for 'table_size' elements. If passing FP argument to the
+  // Reserves table_size cells of space. If passing FP argument to the
   // constructor, this class owns it.
   explicit VectorBiTable(FP *fp = nullptr, size_t table_size = 0) :
       fp_(fp ? fp : new FP()) {
@@ -268,15 +266,15 @@ class VectorBiTable {
     ssize_t fp = (*fp_)(entry);
     if (fp >= fp2id_.size()) fp2id_.resize(fp + 1);
     I &id_ref = fp2id_[fp];
-    if (id_ref == 0) {  // T not found
-      if (insert) {     // store and assign it a new ID
+    if (id_ref == 0) {  // T not found.
+      if (insert) {     // Stores and assigns a new ID.
         id2entry_.push_back(entry);
         id_ref = id2entry_.size();
       } else {
         return -1;
       }
     }
-    return id_ref - 1;  // NB: id_ref = ID + 1
+    return id_ref - 1;  // NB: id_ref = ID + 1.
   }
 
   const T &FindEntry(I s) const { return id2entry_[s]; }
@@ -289,17 +287,14 @@ class VectorBiTable {
   std::unique_ptr<FP> fp_;
   std::vector<I> fp2id_;
   std::vector<T> id2entry_;
-
-  void operator=(const VectorBiTable<I, T, FP> &table);  // disallow
 };
 
-// An implementation using a vector and a compact hash table. The
-// selecting functor S returns true for entries to be hashed in the
-// vector.  The fingerprinting functor FP returns a unique fingerprint
-// for each entry to be hashed in the vector (these need to be
-// suitable for indexing in a vector).  The hash functor H is used
-// when hashing entry into the compact hash table.  If passed to the
-// constructor, ownership is given to this class.
+// An implementation using a vector and a compact hash table. The selecting
+// functor S returns true for entries to be hashed in the vector. The
+// fingerprinting functor FP returns a unique fingerprint for each entry to be
+// hashed in the vector (these need to be suitable for indexing in a vector).
+// The hash functor H is used when hashing entry into the compact hash table.
+// If passed to the constructor, ownership is given to this class.
 template <class I, class T, class S, class FP, class H, HSType HS = HS_DENSE>
 class VectorHashBiTable {
  public:
@@ -323,19 +318,19 @@ class VectorHashBiTable {
   }
 
   I FindId(const T &entry, bool insert = true) {
-    if ((*selector_)(entry)) {  // Use the vector if 'selector_(entry) == true'
+    if ((*selector_)(entry)) {  // Uses the vector if selector_(entry) == true.
       uint64 fp = (*fp_)(entry);
       if (fp2id_.size() <= fp) fp2id_.resize(fp + 1, 0);
-      if (fp2id_[fp] == 0) {  // T not found
-        if (insert) {         // store and assign it a new ID
+      if (fp2id_[fp] == 0) {  // T not found.
+        if (insert) {         // Stores and assigns a new ID.
           id2entry_.push_back(entry);
           fp2id_[fp] = id2entry_.size();
         } else {
           return -1;
         }
       }
-      return fp2id_[fp] - 1;  // NB: assoc_value = ID + 1
-    } else {                  // Use the hash table otherwise.
+      return fp2id_[fp] - 1;  // NB: assoc_value = ID + 1.
+    } else {                  // Uses the hash table otherwise.
       current_entry_ = &entry;
       const auto it = keys_.find(kCurrentKey);
       if (it == keys_.end()) {
@@ -364,12 +359,12 @@ class VectorHashBiTable {
   const H &Hash() const { return *h_; }
 
  private:
-  static const I kCurrentKey;  // -1
-  static const I kEmptyKey;    // -2
+  static constexpr I kCurrentKey = -1;
+  static constexpr I kEmptyKey = -2;
 
   class HashFunc {
    public:
-    HashFunc(const VectorHashBiTable &ht) : ht_(&ht) {}
+    explicit HashFunc(const VectorHashBiTable &ht) : ht_(&ht) {}
 
     size_t operator()(I k) const {
       if (k >= kCurrentKey) {
@@ -385,7 +380,7 @@ class VectorHashBiTable {
 
   class HashEqual {
    public:
-    HashEqual(const VectorHashBiTable &ht) : ht_(&ht) {}
+    explicit HashEqual(const VectorHashBiTable &ht) : ht_(&ht) {}
 
     bool operator()(I k1, I k2) const {
       if (k1 >= kCurrentKey && k2 >= kCurrentKey) {
@@ -399,43 +394,41 @@ class VectorHashBiTable {
     const VectorHashBiTable *ht_;
   };
 
-  typedef HashSet<I, HashFunc, HashEqual, HS> KeyHashSet;
+  using KeyHashSet = HashSet<I, HashFunc, HashEqual, HS>;
 
   const T &Key2Entry(I k) const {
-    if (k == kCurrentKey)
+    if (k == kCurrentKey) {
       return *current_entry_;
-    else
+    } else {
       return id2entry_[k];
+    }
   }
 
-  std::unique_ptr<S> selector_;  // Returns true if entry hashed into vector.
-  std::unique_ptr<FP> fp_;       // Fingerprint used when hashing into vector.
-  std::unique_ptr<H> h_;         // Hash fnc used when hashing into hash_set.
+  std::unique_ptr<S> selector_;  // True if entry hashed into vector.
+  std::unique_ptr<FP> fp_;       // Fingerprint used for hashing into vector.
+  std::unique_ptr<H> h_;         // Hash funcion used for hashing into hash_set.
 
-  std::vector<T> id2entry_;  // Maps state IDs to entry
-  std::vector<I> fp2id_;     // Maps entry fingerprints to IDs
+  std::vector<T> id2entry_;  // Maps state IDs to entry.
+  std::vector<I> fp2id_;     // Maps entry fingerprints to IDs.
 
-  // Compact implementation of the hash table mapping entrys to
-  // state IDs using the hash function 'h_'
+  // Compact implementation of the hash table mapping entries to state IDs
+  // using the hash function h_.
   HashFunc hash_func_;
   HashEqual hash_equal_;
   KeyHashSet keys_;
   const T *current_entry_;
-
-  // disallow
-  void operator=(const VectorHashBiTable<I, T, S, FP, H, HS> &table);
 };
 
 template <class I, class T, class S, class FP, class H, HSType HS>
-const I VectorHashBiTable<I, T, S, FP, H, HS>::kCurrentKey = -1;
+constexpr I VectorHashBiTable<I, T, S, FP, H, HS>::kCurrentKey;
 
 template <class I, class T, class S, class FP, class H, HSType HS>
-const I VectorHashBiTable<I, T, S, FP, H, HS>::kEmptyKey = -3;
+constexpr I VectorHashBiTable<I, T, S, FP, H, HS>::kEmptyKey;
 
-// An implementation using a hash map for the entry to ID
-// mapping. This version permits erasing of arbitrary states.  The
-// entry T must have == defined and its default constructor must
-// produce a entry that will never be seen. F is the hash function.
+// An implementation using a hash map for the entry to ID mapping. This version
+// permits erasing of arbitrary states. The entry T must have == defined and
+// its default constructor must produce a entry that will never be seen. F is
+// the hash function.
 template <class I, class T, class F>
 class ErasableBiTable {
  public:
@@ -443,15 +436,15 @@ class ErasableBiTable {
 
   I FindId(const T &entry, bool insert = true) {
     I &id_ref = entry2id_[entry];
-    if (id_ref == 0) {  // T not found
-      if (insert) {     // store and assign it a new ID
+    if (id_ref == 0) {  // T not found.
+      if (insert) {     // Stores and assigns a new ID.
         id2entry_.push_back(entry);
         id_ref = id2entry_.size() + first_;
       } else {
         return -1;
       }
     }
-    return id_ref - 1;  // NB: id_ref = ID + 1
+    return id_ref - 1;  // NB: id_ref = ID + 1.
   }
 
   const T &FindEntry(I s) const { return id2entry_[s - first_]; }
@@ -459,10 +452,9 @@ class ErasableBiTable {
   I Size() const { return id2entry_.size(); }
 
   void Erase(I s) {
-    T &entry = id2entry_[s - first_];
-    const auto it = entry2id_.find(entry);
-    entry2id_.erase(it);
-    id2entry_[s - first_] = empty_entry_;
+    auto &ref = id2entry_[s - first_];
+    entry2id_.erase(ref);
+    ref = empty_entry_;
     while (!id2entry_.empty() && id2entry_.front() == empty_entry_) {
       id2entry_.pop_front();
       ++first_;
@@ -473,12 +465,9 @@ class ErasableBiTable {
   std::unordered_map<T, I, F> entry2id_;
   std::deque<T> id2entry_;
   const T empty_entry_;
-  I first_;  // I of first element in the deque;
-
-  // disallow
-  void operator=(const ErasableBiTable<I, T, F> &table);  // disallow
+  I first_;  // I of first element in the deque.
 };
 
 }  // namespace fst
 
-#endif  // FST_LIB_BI_TABLE_H__
+#endif  // FST_LIB_BI_TABLE_H_
