@@ -16,26 +16,25 @@
 namespace fst {
 
 // Finds and returns connected components. Use with Visit().
-template <class A>
+template <class Arc>
 class CcVisitor {
  public:
-  typedef A Arc;
-  typedef typename Arc::Weight Weight;
-  typedef typename A::StateId StateId;
+  using Weight = typename Arc::Weight;
+  using StateId = typename Arc::StateId;
 
   // cc[i]: connected component number for state i.
   explicit CcVisitor(std::vector<StateId> *cc)
       : comps_(new UnionFind<StateId>(0, kNoStateId)), cc_(cc), nstates_(0) {}
 
   // comps: connected components equiv classes.
-  CcVisitor(UnionFind<StateId> *comps) : comps_(comps), cc_(0), nstates_(0) {}
+  explicit CcVisitor(UnionFind<StateId> *comps)
+      : comps_(comps), cc_(nullptr), nstates_(0) {}
 
   ~CcVisitor() {
-    if (cc_)  // own comps_?
-      delete comps_;
+    if (cc_) delete comps_;
   }
 
-  void InitVisit(const Fst<A> &fst) {}
+  void InitVisit(const Fst<Arc> &fst) {}
 
   bool InitState(StateId s, StateId root) {
     ++nstates_;
@@ -43,18 +42,18 @@ class CcVisitor {
     return true;
   }
 
-  bool WhiteArc(StateId s, const A &arc) {
+  bool WhiteArc(StateId s, const Arc &arc) {
     comps_->MakeSet(arc.nextstate);
     comps_->Union(s, arc.nextstate);
     return true;
   }
 
-  bool GreyArc(StateId s, const A &arc) {
+  bool GreyArc(StateId s, const Arc &arc) {
     comps_->Union(s, arc.nextstate);
     return true;
   }
 
-  bool BlackArc(StateId s, const A &arc) {
+  bool BlackArc(StateId s, const Arc &arc) {
     comps_->Union(s, arc.nextstate);
     return true;
   }
@@ -65,40 +64,39 @@ class CcVisitor {
     if (cc_) GetCcVector(cc_);
   }
 
-  // cc[i]: connected component number for state i.
   // Returns number of components.
+  // cc[i]: connected component number for state i.
   int GetCcVector(std::vector<StateId> *cc) {
     cc->clear();
     cc->resize(nstates_, kNoStateId);
     StateId ncomp = 0;
-    for (StateId i = 0; i < nstates_; ++i) {
-      StateId rep = comps_->FindSet(i);
-      StateId &comp = (*cc)[rep];
+    for (StateId s = 0; s < nstates_; ++s) {
+      const auto rep = comps_->FindSet(s);
+      auto &comp = (*cc)[rep];
       if (comp == kNoStateId) {
         comp = ncomp;
         ++ncomp;
       }
-      (*cc)[i] = comp;
+      (*cc)[s] = comp;
     }
     return ncomp;
   }
 
  private:
-  UnionFind<StateId> *comps_;  // Components
-  std::vector<StateId> *cc_;   // State's cc number
-  StateId nstates_;            // State count
+  UnionFind<StateId> *comps_;  // Components.
+  std::vector<StateId> *cc_;   // State's cc number.
+  StateId nstates_;            // State count.
 };
 
 // Finds and returns strongly-connected components, accessible and
 // coaccessible states and related properties. Uses Tarjan's single
 // DFS SCC algorithm (see Aho, et al, "Design and Analysis of Computer
 // Algorithms", 189pp). Use with DfsVisit();
-template <class A>
+template <class Arc>
 class SccVisitor {
  public:
-  typedef A Arc;
-  typedef typename A::Weight Weight;
-  typedef typename A::StateId StateId;
+  using StateId = typename Arc::StateId;
+  using Weight = typename Arc::Weight;
 
   // scc[i]: strongly-connected component number for state i.
   //   SCC numbers will be in topological order for acyclic input.
@@ -110,69 +108,75 @@ class SccVisitor {
   SccVisitor(std::vector<StateId> *scc, std::vector<bool> *access,
              std::vector<bool> *coaccess, uint64 *props)
       : scc_(scc), access_(access), coaccess_(coaccess), props_(props) {}
-  SccVisitor(uint64 *props)
-      : scc_(0), access_(0), coaccess_(0), props_(props) {}
+  explicit SccVisitor(uint64 *props)
+      : scc_(nullptr), access_(nullptr), coaccess_(nullptr), props_(props) {}
 
-  void InitVisit(const Fst<A> &fst);
+  void InitVisit(const Fst<Arc> &fst);
 
   bool InitState(StateId s, StateId root);
 
-  bool TreeArc(StateId s, const A &arc) { return true; }
+  bool TreeArc(StateId s, const Arc &arc) { return true; }
 
-  bool BackArc(StateId s, const A &arc) {
-    StateId t = arc.nextstate;
+  bool BackArc(StateId s, const Arc &arc) {
+    const auto t = arc.nextstate;
     if ((*dfnumber_)[t] < (*lowlink_)[s]) (*lowlink_)[s] = (*dfnumber_)[t];
     if ((*coaccess_)[t]) (*coaccess_)[s] = true;
     *props_ |= kCyclic;
     *props_ &= ~kAcyclic;
-    if (arc.nextstate == start_) {
+    if (t == start_) {
       *props_ |= kInitialCyclic;
       *props_ &= ~kInitialAcyclic;
     }
     return true;
   }
 
-  bool ForwardOrCrossArc(StateId s, const A &arc) {
-    StateId t = arc.nextstate;
+  bool ForwardOrCrossArc(StateId s, const Arc &arc) {
+    const auto t = arc.nextstate;
     if ((*dfnumber_)[t] < (*dfnumber_)[s] /* cross edge */ && (*onstack_)[t] &&
-        (*dfnumber_)[t] < (*lowlink_)[s])
+        (*dfnumber_)[t] < (*lowlink_)[s]) {
       (*lowlink_)[s] = (*dfnumber_)[t];
+    }
     if ((*coaccess_)[t]) (*coaccess_)[s] = true;
     return true;
   }
 
-  void FinishState(StateId s, StateId p, const A *);
+  // Last argument always ignored, but required by the interface.
+  void FinishState(StateId state, StateId p, const Arc *);
 
   void FinishVisit() {
-    // Numbers SCC's in topological order when acyclic.
-    if (scc_)
-      for (StateId i = 0; i < scc_->size(); ++i)
-        (*scc_)[i] = nscc_ - 1 - (*scc_)[i];
+    // Numbers SCCs in topological order when acyclic.
+    if (scc_) {
+      for (StateId s = 0; s < scc_->size(); ++s) {
+        (*scc_)[s] = nscc_ - 1 - (*scc_)[s];
+      }
+    }
     if (coaccess_internal_) delete coaccess_;
-    delete dfnumber_;
-    delete lowlink_;
-    delete onstack_;
-    delete scc_stack_;
+    dfnumber_.reset();
+    lowlink_.reset();
+    onstack_.reset();
+    scc_stack_.reset();
   }
 
  private:
-  std::vector<StateId> *scc_;    // State's scc number
-  std::vector<bool> *access_;    // State's accessibility
-  std::vector<bool> *coaccess_;  // State's coaccessibility
+  std::vector<StateId> *scc_;    // State's scc number.
+  std::vector<bool> *access_;    // State's accessibility.
+  std::vector<bool> *coaccess_;  // State's coaccessibility.
   uint64 *props_;
-  const Fst<A> *fst_;
+  const Fst<Arc> *fst_;
   StateId start_;
-  StateId nstates_;  // State count
-  StateId nscc_;     // SCC count
+  StateId nstates_;  // State count.
+  StateId nscc_;     // SCC count.
   bool coaccess_internal_;
-  std::vector<StateId> *dfnumber_;   // state discovery times
-  std::vector<StateId> *lowlink_;    // lowlink[s] == dfnumber[s] => SCC root
-  std::vector<bool> *onstack_;       // is a state on the SCC stack
-  std::vector<StateId> *scc_stack_;  // SCC stack (w/ random access)
+  std::unique_ptr<std::vector<StateId>> dfnumber_;  // State discovery times.
+  std::unique_ptr<std::vector<StateId>>
+      lowlink_;  // lowlink[state] == dfnumber[state] => SCC root
+  std::unique_ptr<std::vector<bool>> onstack_;  // Is a state on the SCC stack?
+  std::unique_ptr<std::vector<StateId>>
+      scc_stack_;  // SCC stack, with random access.
 };
 
-template <class A>
-inline void SccVisitor<A>::InitVisit(const Fst<A> &fst) {
+template <class Arc>
+inline void SccVisitor<Arc>::InitVisit(const Fst<Arc> &fst) {
   if (scc_) scc_->clear();
   if (access_) access_->clear();
   if (coaccess_) {
@@ -188,14 +192,14 @@ inline void SccVisitor<A>::InitVisit(const Fst<A> &fst) {
   start_ = fst.Start();
   nstates_ = 0;
   nscc_ = 0;
-  dfnumber_ = new std::vector<StateId>;
-  lowlink_ = new std::vector<StateId>;
-  onstack_ = new std::vector<bool>;
-  scc_stack_ = new std::vector<StateId>;
+  dfnumber_.reset(new std::vector<StateId>());
+  lowlink_.reset(new std::vector<StateId>());
+  onstack_.reset(new std::vector<bool>());
+  scc_stack_.reset(new std::vector<StateId>());
 }
 
-template <class A>
-inline bool SccVisitor<A>::InitState(StateId s, StateId root) {
+template <class Arc>
+inline bool SccVisitor<Arc>::InitState(StateId s, StateId root) {
   scc_stack_->push_back(s);
   while (dfnumber_->size() <= s) {
     if (scc_) scc_->push_back(-1);
@@ -219,12 +223,12 @@ inline bool SccVisitor<A>::InitState(StateId s, StateId root) {
   return true;
 }
 
-template <class A>
-inline void SccVisitor<A>::FinishState(StateId s, StateId p, const A *) {
+template <class Arc>
+inline void SccVisitor<Arc>::FinishState(StateId s, StateId p, const Arc *) {
   if (fst_->Final(s) != Weight::Zero()) (*coaccess_)[s] = true;
-  if ((*dfnumber_)[s] == (*lowlink_)[s]) {  // root of new SCC
+  if ((*dfnumber_)[s] == (*lowlink_)[s]) {  // Root of new SCC.
     bool scc_coaccess = false;
-    size_t i = scc_stack_->size();
+    auto i = scc_stack_->size();
     StateId t;
     do {
       t = (*scc_stack_)[--i];
@@ -249,52 +253,52 @@ inline void SccVisitor<A>::FinishState(StateId s, StateId p, const A *) {
   }
 }
 
-// Trims an FST, removing states and arcs that are not on successful
-// paths. This version modifies its input.
+// Trims an FST, removing states and arcs that are not on successful paths.
+// This version modifies its input.
 //
 // Complexity:
-// - Time:  O(V + E)
-// - Space: O(V + E)
+//
+//   Time:  O(V + E)
+//   Space: O(V + E)
+//
 // where V = # of states and E = # of arcs.
 template <class Arc>
 void Connect(MutableFst<Arc> *fst) {
-  typedef typename Arc::StateId StateId;
-
+  using StateId = typename Arc::StateId;
   std::vector<bool> access;
   std::vector<bool> coaccess;
   uint64 props = 0;
-  SccVisitor<Arc> scc_visitor(0, &access, &coaccess, &props);
+  SccVisitor<Arc> scc_visitor(nullptr, &access, &coaccess, &props);
   DfsVisit(*fst, &scc_visitor);
   std::vector<StateId> dstates;
-  for (StateId s = 0; s < access.size(); ++s)
+  for (StateId s = 0; s < access.size(); ++s) {
     if (!access[s] || !coaccess[s]) dstates.push_back(s);
+  }
   fst->DeleteStates(dstates);
   fst->SetProperties(kAccessible | kCoAccessible, kAccessible | kCoAccessible);
 }
 
-// Returns an acyclic FST where each SCC in the input FST has been
-// condensed to a single state with transitions between SCCs retained
-// and within SCCs dropped.  Also returns the mapping from an input
-// state 's' to an output state 'scc[s]'.
+// Returns an acyclic FST where each SCC in the input FST has been condensed to
+// a single state with transitions between SCCs retained and within SCCs
+// dropped. Also populates 'scc' with a mapping from input to output states.
 template <class Arc>
 void Condense(const Fst<Arc> &ifst, MutableFst<Arc> *ofst,
               std::vector<typename Arc::StateId> *scc) {
-  typedef typename Arc::StateId StateId;
-  typedef typename Arc::Weight Weight;
-
+  using StateId = typename Arc::StateId;
   ofst->DeleteStates();
   uint64 props = 0;
-  SccVisitor<Arc> scc_visitor(scc, 0, 0, &props);
+  SccVisitor<Arc> scc_visitor(scc, nullptr, nullptr, &props);
   DfsVisit(ifst, &scc_visitor);
   for (StateId s = 0; s < scc->size(); ++s) {
-    StateId c = (*scc)[s];
+    const auto c = (*scc)[s];
     while (c >= ofst->NumStates()) ofst->AddState();
     if (s == ifst.Start()) ofst->SetStart(c);
-    Weight final = ifst.Final(s);
-    if (final != Weight::Zero()) ofst->SetFinal(c, Plus(ofst->Final(c), final));
+    const auto weight = ifst.Final(s);
+    if (weight != Arc::Weight::Zero())
+      ofst->SetFinal(c, Plus(ofst->Final(c), weight));
     for (ArcIterator<Fst<Arc>> aiter(ifst, s); !aiter.Done(); aiter.Next()) {
-      Arc arc = aiter.Value();
-      StateId nextc = (*scc)[arc.nextstate];
+      auto arc = aiter.Value();
+      const auto nextc = (*scc)[arc.nextstate];
       if (nextc != c) {
         while (nextc >= ofst->NumStates()) ofst->AddState();
         arc.nextstate = nextc;
