@@ -6,11 +6,10 @@
 #ifndef FST_LIB_DISAMBIGUATE_H_
 #define FST_LIB_DISAMBIGUATE_H_
 
-#include <algorithm>
-#include <climits>
+#include <list>
 #include <map>
 #include <set>
-#include <string>
+#include <utility>
 #include <vector>
 
 #include <fst/arcsort.h>
@@ -30,81 +29,80 @@ namespace fst {
 
 template <class Arc>
 struct DisambiguateOptions : public DeterminizeOptions<Arc> {
-  typedef typename Arc::StateId StateId;
-  typedef typename Arc::Weight Weight;
-  typedef typename Arc::Label Label;
+  using Label = typename Arc::Label;
+  using StateId = typename Arc::StateId;
+  using Weight = typename Arc::Weight;
 
-  explicit DisambiguateOptions(float d = kDelta, Weight w = Weight::Zero(),
-                               StateId n = kNoStateId, Label l = 0)
-      : DeterminizeOptions<Arc>(d, w, n, l, DETERMINIZE_FUNCTIONAL) {}
+  explicit DisambiguateOptions(float delta = kDelta,
+                               Weight weight = Weight::Zero(),
+                               StateId n = kNoStateId, Label label = 0)
+      : DeterminizeOptions<Arc>(delta, std::move(weight), n, label,
+                                DETERMINIZE_FUNCTIONAL) {}
 };
 
-// A determinize filter based on a subset element relation.
-// The relation is assumed to be reflexive and symmetric.
-template <class Arc, class R>
+namespace internal {
+
+// A determinization filter based on a subset element relation. The relation is
+// assumed to be reflexive and symmetric.
+template <class Arc, class Relation>
 class RelationDeterminizeFilter {
  public:
-  typedef typename Arc::StateId StateId;
-  typedef typename Arc::Label Label;
-  typedef typename Arc::Weight Weight;
+  using Label = typename Arc::Label;
+  using StateId = typename Arc::StateId;
+  using Weight = typename Arc::Weight;
 
-  typedef IntegerFilterState<StateId> FilterState;
-  typedef DeterminizeStateTuple<Arc, FilterState> StateTuple;
-  typedef typename StateTuple::Subset Subset;
-  typedef typename StateTuple::Element Element;
-  typedef std::multimap<Label, DeterminizeArc<StateTuple>> LabelMap;
+  using FilterState = IntegerFilterState<StateId>;
+  using StateTuple = DeterminizeStateTuple<Arc, FilterState>;
+  using Subset = typename StateTuple::Subset;
+  using Element = typename StateTuple::Element;
+  using LabelMap = std::multimap<Label, DeterminizeArc<StateTuple>>;
 
-  // This is needed e.g. to go into the gallic domain for transducers.
-  // No need to rebind the relation since its use here only depends
-  // on the state Ids.
+  // This is needed (e.g.) to go into the gallic domain for transducers; there
+  // is no need to rebind the relation since its use here only depends on the
+  // state IDs.
   template <class A>
   struct rebind {
-    typedef RelationDeterminizeFilter<A, R> other;
+    using Other = RelationDeterminizeFilter<A, Relation>;
   };
 
-  RelationDeterminizeFilter(const Fst<Arc> &fst)
-      : fst_(fst.Copy()), r_(new R()), s_(kNoStateId), head_(0) {}
+  explicit RelationDeterminizeFilter(const Fst<Arc> &fst)
+      : fst_(fst.Copy()), r_(new Relation()), s_(kNoStateId), head_(nullptr) {}
 
   // Ownership of the relation is given to this class.
-  RelationDeterminizeFilter(const Fst<Arc> &fst, R *r)
+  RelationDeterminizeFilter(const Fst<Arc> &fst, Relation *r)
       : fst_(fst.Copy()), r_(r), s_(kNoStateId), head_(0) {}
 
-  // Ownership of the relation is given to this class. Returns head states.
-  RelationDeterminizeFilter(const Fst<Arc> &fst, R *r,
+  // Ownership of the relation is given to this class.
+  RelationDeterminizeFilter(const Fst<Arc> &fst, Relation *r,
                             std::vector<StateId> *head)
       : fst_(fst.Copy()), r_(r), s_(kNoStateId), head_(head) {}
 
-  // This is needed e.g. to go into the gallic domain for transducers.
+  // This is needed, e.g., to go into the gallic domain for transducers.
   // Ownership of the templated filter argument is given to this class.
-  template <class F>
-  RelationDeterminizeFilter(const Fst<Arc> &fst, F *filter)
+  template <class Filter>
+  RelationDeterminizeFilter(const Fst<Arc> &fst, Filter *filter)
       : fst_(fst.Copy()),
-        r_(new R(filter->GetRelation())),
+        r_(new Relation(filter->GetRelation())),
         s_(kNoStateId),
         head_(filter->GetHeadStates()) {
     delete filter;
   }
 
-  // Copy ctr. The FST can be passed if it has been e.g. (deep) copied.
-  explicit RelationDeterminizeFilter(
-      const RelationDeterminizeFilter<Arc, R> &filter, const Fst<Arc> *fst = 0)
+  // Copy constructor; the FST can be passed if it has been deep-copied.
+  RelationDeterminizeFilter(const RelationDeterminizeFilter &filter,
+                            const Fst<Arc> *fst = nullptr)
       : fst_(fst ? fst->Copy() : filter.fst_->Copy()),
-        r_(new R(*filter.r_)),
+        r_(new Relation(*filter.r_)),
         s_(kNoStateId),
         head_() {}
 
-  ~RelationDeterminizeFilter() {
-    delete fst_;
-    delete r_;
-  }
-
   FilterState Start() const { return FilterState(fst_->Start()); }
 
   void SetState(StateId s, const StateTuple &tuple) {
     if (s_ != s) {
       s_ = s;
       tuple_ = &tuple;
-      StateId head = tuple.filter_state.GetState();
+      const auto head = tuple.filter_state.GetState();
       is_final_ = fst_->Final(head) != Weight::Zero();
       if (head_) {
         if (head_->size() <= s) head_->resize(s + 1, kNoStateId);
@@ -113,12 +111,12 @@ class RelationDeterminizeFilter {
     }
   }
 
-  // Filters transition, possibly modifying label map. Returns
-  // true if arc is added to label map.
+  // Filters transition, possibly modifying label map. Returns true if arc is
+  // added to label map.
   bool FilterArc(const Arc &arc, const Element &src_element,
                  const Element &dest_element, LabelMap *label_map) const;
 
-  // Filters super-final transition, returning new final weight
+  // Filters super-final transition, returning new final weight.
   Weight FilterFinal(const Weight final_weight, const Element &element) const {
     return is_final_ ? final_weight : Weight::Zero();
   }
@@ -127,37 +125,34 @@ class RelationDeterminizeFilter {
     return props & ~(kIDeterministic | kODeterministic);
   }
 
-  const R &GetRelation() { return *r_; }
+  const Relation &GetRelation() { return *r_; }
+
   std::vector<StateId> *GetHeadStates() { return head_; }
 
  private:
-  // Pairs arc labels with state tuples with possible heads and
-  // empty subsets.
+  // Pairs arc labels with state tuples with possible heads and empty subsets.
   void InitLabelMap(LabelMap *label_map) const;
 
-  Fst<Arc> *fst_;  // Input FST
-  R *r_;           // Relation compatible with the inverse trans. function
-  StateId s_;      // Current state
-  const StateTuple *tuple_;  // Current tuple
-  bool is_final_;            // Is the current head state final?
-  std::vector<StateId> *head_;  // Head state for a given state.
-
-  void operator=(const RelationDeterminizeFilter<Arc, R> &filt);  // disallow
+  std::unique_ptr<Fst<Arc>> fst_;  // Input FST.
+  std::unique_ptr<Relation> r_;    // Relation compatible with inv. trans. fnc.
+  StateId s_;                      // Current state.
+  const StateTuple *tuple_;        // Current tuple.
+  bool is_final_;                  // Is the current head state final?
+  std::vector<StateId> *head_;     // Head state for a given state,
+                                   // owned by the Disambiguator.
 };
 
-template <class Arc, class R>
-bool RelationDeterminizeFilter<Arc, R>::FilterArc(const Arc &arc,
-                                                  const Element &src_element,
-                                                  const Element &dest_element,
-                                                  LabelMap *label_map) const {
+template <class Arc, class Relation>
+bool RelationDeterminizeFilter<Arc, Relation>::FilterArc(
+    const Arc &arc, const Element &src_element, const Element &dest_element,
+    LabelMap *label_map) const {
   bool added = false;
   if (label_map->empty()) InitLabelMap(label_map);
-
   // Adds element to state tuple if element state is related to tuple head.
   for (auto liter = label_map->lower_bound(arc.ilabel);
        liter != label_map->end() && liter->first == arc.ilabel; ++liter) {
-    StateTuple *dest_tuple = liter->second.dest_tuple;
-    StateId dest_head = dest_tuple->filter_state.GetState();
+    auto *dest_tuple = liter->second.dest_tuple;
+    const auto dest_head = dest_tuple->filter_state.GetState();
     if ((*r_)(dest_element.state_id, dest_head)) {
       dest_tuple->subset.push_front(dest_element);
       added = true;
@@ -166,21 +161,20 @@ bool RelationDeterminizeFilter<Arc, R>::FilterArc(const Arc &arc,
   return added;
 }
 
-template <class Arc, class R>
-void RelationDeterminizeFilter<Arc, R>::InitLabelMap(
+template <class Arc, class Relation>
+void RelationDeterminizeFilter<Arc, Relation>::InitLabelMap(
     LabelMap *label_map) const {
-  StateId src_head = tuple_->filter_state.GetState();
+  const auto src_head = tuple_->filter_state.GetState();
   Label label = kNoLabel;
   StateId nextstate = kNoStateId;
   for (ArcIterator<Fst<Arc>> aiter(*fst_, src_head); !aiter.Done();
        aiter.Next()) {
-    const Arc &arc = aiter.Value();
-    if (arc.ilabel == label && arc.nextstate == nextstate)
-      continue;  // multiarc
+    const auto &arc = aiter.Value();
+    // Continues if multiarc.
+    if (arc.ilabel == label && arc.nextstate == nextstate) continue;
     DeterminizeArc<StateTuple> det_arc(arc);
     det_arc.dest_tuple->filter_state = FilterState(arc.nextstate);
-    std::pair<Label, DeterminizeArc<StateTuple>> pr(arc.ilabel, det_arc);
-    label_map->insert(pr);
+    label_map->insert(std::make_pair(arc.ilabel, det_arc));
     label = arc.ilabel;
     nextstate = arc.nextstate;
   }
@@ -190,14 +184,15 @@ void RelationDeterminizeFilter<Arc, R>::InitLabelMap(
 template <class Arc>
 class Disambiguator {
  public:
-  typedef typename Arc::StateId StateId;
-  typedef typename Arc::Weight Weight;
-  typedef typename Arc::Label Label;
-  // Ids arc with state id and arc position. Arc pos of -1 means
-  // final (super-final transition).
-  typedef std::pair<StateId, ssize_t> ArcId;
+  using Label = typename Arc::Label;
+  using StateId = typename Arc::StateId;
+  using Weight = typename Arc::Weight;
 
-  Disambiguator() : candidates_(0), merge_(0), error_(false) {}
+  // IDs arcs with state ID and arc position. Arc position -1 indicates final
+  // (super-final transition).
+  using ArcId = std::pair<StateId, ssize_t>;
+
+  Disambiguator() : error_(false) {}
 
   void Disambiguate(
       const Fst<Arc> &ifst, MutableFst<Arc> *ofst,
@@ -215,11 +210,11 @@ class Disambiguator {
   }
 
  private:
-  // Compare class for comparing input labels and next states of arcs.
-  // This sort order facilitates the predisambiguation.
+  // Comparison functor for comparing input labels and next states of arcs. This
+  // sort order facilitates the predisambiguation.
   class ArcCompare {
    public:
-    bool operator()(Arc arc1, Arc arc2) const {
+    bool operator()(const Arc &arc1, const Arc &arc2) const {
       return arc1.ilabel < arc2.ilabel ||
              (arc1.ilabel == arc2.ilabel && arc1.nextstate < arc2.nextstate);
     }
@@ -230,30 +225,25 @@ class Disambiguator {
     }
   };
 
-  // Compare class for comparing transitions represented by their arc ID.
+  // Comparison functor for comparing transitions represented by their arc ID.
   // This sort order facilitates ambiguity detection.
   class ArcIdCompare {
    public:
     explicit ArcIdCompare(const std::vector<StateId> &head) : head_(head) {}
 
     bool operator()(const ArcId &a1, const ArcId &a2) const {
-      StateId src1 = a1.first;
-      StateId src2 = a2.first;
-      StateId pos1 = a1.second;
-      StateId pos2 = a2.second;
-      StateId head1 = head_[src1];
-      StateId head2 = head_[src2];
-
-      // Sort first by src head state
+      // Sort first by source head state...
+      const auto src1 = a1.first;
+      const auto src2 = a2.first;
+      const auto head1 = head_[src1];
+      const auto head2 = head_[src2];
       if (head1 < head2) return true;
       if (head2 < head1) return false;
-
-      // ... then by src state
+      // ...then by source state...
       if (src1 < src2) return true;
       if (src2 < src1) return false;
-
-      // ... then by position
-      return pos1 < pos2;
+      // ...then by position.
+      return a1.second < a2.second;
     }
 
    private:
@@ -263,41 +253,37 @@ class Disambiguator {
   // A relation that determines if two states share a common future.
   class CommonFuture {
    public:
-    typedef GenericComposeStateTable<Arc, TrivialFilterState> T;
-    typedef typename T::StateTuple StateTuple;
+    using StateTable = GenericComposeStateTable<Arc, TrivialFilterState>;
+    using StateTuple = typename StateTable::StateTuple;
 
-    // Needed for compilation with DeterminizeRelationFilter
+    // Needed for compilation with DeterminizeRelationFilter.
     CommonFuture() {
-      FSTERROR() << "Disambiguate::CommonFuture: Fst not provided";
+      FSTERROR() << "Disambiguate::CommonFuture: FST not provided";
     }
 
     explicit CommonFuture(const Fst<Arc> &ifst) {
-      typedef Matcher<Fst<Arc>> M;
+      using M = Matcher<Fst<Arc>>;
       ComposeFstOptions<Arc, M, NullComposeFilter<M>> opts;
-
       // Ensures composition is between acceptors.
-      bool trans = ifst.Properties(kNotAcceptor, true);
-      const Fst<Arc> *fsa =
+      const bool trans = ifst.Properties(kNotAcceptor, true);
+      const auto *fsa =
           trans ? new ProjectFst<Arc>(ifst, PROJECT_INPUT) : &ifst;
-
-      opts.state_table = new T(*fsa, *fsa);
-      ComposeFst<Arc> cfst(*fsa, *fsa, opts);
+      opts.state_table = new StateTable(*fsa, *fsa);
+      const ComposeFst<Arc> cfst(*fsa, *fsa, opts);
       std::vector<bool> coaccess;
       uint64 props = 0;
-      SccVisitor<Arc> scc_visitor(0, 0, &coaccess, &props);
+      SccVisitor<Arc> scc_visitor(nullptr, nullptr, &coaccess, &props);
       DfsVisit(cfst, &scc_visitor);
       for (StateId s = 0; s < coaccess.size(); ++s) {
         if (coaccess[s]) {
-          const StateTuple &tuple = opts.state_table->Tuple(s);
-          related_.insert(tuple.StatePair());
+          related_.insert(opts.state_table->Tuple(s).StatePair());
         }
       }
       if (trans) delete fsa;
     }
 
     bool operator()(const StateId s1, StateId s2) const {
-      std::pair<StateId, StateId> pr(s1, s2);
-      return related_.count(pr) > 0;
+      return related_.count(std::make_pair(s1, s2)) > 0;
     }
 
    private:
@@ -307,22 +293,28 @@ class Disambiguator {
     std::set<std::pair<StateId, StateId>> related_;
   };
 
-  typedef std::multimap<ArcId, ArcId, ArcIdCompare> ArcIdMap;
+  using ArcIdMap = std::multimap<ArcId, ArcId, ArcIdCompare>;
 
-  // Returns the arc corresponding to ArcId a
-  static Arc GetArc(const Fst<Arc> &fst, ArcId a) {
-    if (a.second == -1) {  // returns super-final transition
-      Arc arc(kNoLabel, kNoLabel, fst.Final(a.first), kNoStateId);
-      return arc;
+  // Inserts candidate into the arc ID map.
+  inline void InsertCandidate(StateId s1, StateId s2, const ArcId &a1,
+                              const ArcId &a2) {
+    candidates_->insert(head_[s1] > head_[s2] ? std::make_pair(a1, a2)
+                                              : std::make_pair(a2, a1));
+  }
+
+  // Returns the arc corresponding to ArcId a.
+  static Arc GetArc(const Fst<Arc> &fst, ArcId aid) {
+    if (aid.second == -1) {  // Returns super-final transition.
+      return Arc(kNoLabel, kNoLabel, fst.Final(aid.first), kNoStateId);
     } else {
-      ArcIterator<Fst<Arc>> aiter(fst, a.first);
-      aiter.Seek(a.second);
+      ArcIterator<Fst<Arc>> aiter(fst, aid.first);
+      aiter.Seek(aid.second);
       return aiter.Value();
     }
   }
 
-  // Outputs an equivalent FST whose states are subsets of states
-  // that have a future path in common.
+  // Outputs an equivalent FST whose states are subsets of states that have a
+  // future path in common.
   void PreDisambiguate(const ExpandedFst<Arc> &ifst, MutableFst<Arc> *ofst,
                        const DisambiguateOptions<Arc> &opts);
 
@@ -337,52 +329,57 @@ class Disambiguator {
   // Marks ambiguous transitions to be removed.
   void MarkAmbiguities();
 
-  // Deletes spurious ambiguous transitions (due to quantization)
+  // Deletes spurious ambiguous transitions (due to quantization).
   void RemoveSplits(MutableFst<Arc> *ofst);
 
   // Deletes actual ambiguous transitions.
   void RemoveAmbiguities(MutableFst<Arc> *ofst);
 
-  // States s1 and s2 are in this relation iff there is a path
-  // from the initial state to s1 that has the same label as some path
-  // from the initial state to s2.  We store only state pairs s1, s2
-  // such that s1 <= s2.
+  // States s1 and s2 are in this relation iff there is a path from the initial
+  // state to s1 that has the same label as some path from the initial state to
+  // s2. We store only state pairs s1, s2 such that s1 <= s2.
   std::set<std::pair<StateId, StateId>> coreachable_;
-  // Queue of disambiguation-related states to be processed. We store
-  // only state pairs s1, s2 such that s1 <= s2.
+
+  // Queue of disambiguation-related states to be processed. We store only
+  // state pairs s1, s2 such that s1 <= s2.
   std::list<std::pair<StateId, StateId>> queue_;
+
   // Head state in the pre-disambiguation for a given state.
   std::vector<StateId> head_;
-  // Maps from a candidate ambiguous arc A to each ambiguous candidate arc
-  // B with the same label and destination state as A, whose source state s'
-  // is coreachable with the source state s of A, and for which
-  // head(s') < head(s).
-  ArcIdMap *candidates_;
+
+  // Maps from a candidate ambiguous arc A to each ambiguous candidate arc B
+  // with the same label and destination state as A, whose source state s' is
+  // coreachable with the source state s of A, and for which head(s') < head(s).
+  std::unique_ptr<ArcIdMap> candidates_;
+
   // Set of ambiguous transitions to be removed.
   std::set<ArcId> ambiguous_;
+
   // States to merge due to quantization issues.
-  UnionFind<StateId> *merge_;
+  std::unique_ptr<UnionFind<StateId>> merge_;
   // Marks error condition.
   bool error_;
-  DISALLOW_COPY_AND_ASSIGN(Disambiguator);
+
+  Disambiguator(const Disambiguator &) = delete;
+  Disambiguator &operator=(const Disambiguator &) = delete;
 };
 
 template <class Arc>
 void Disambiguator<Arc>::PreDisambiguate(const ExpandedFst<Arc> &ifst,
                                          MutableFst<Arc> *ofst,
                                          const DisambiguateOptions<Arc> &opts) {
-  typedef DefaultCommonDivisor<Weight> Div;
-  typedef RelationDeterminizeFilter<Arc, CommonFuture> Filt;
-
-  // Subset elements with states s1 and s2 resp. are in this relation
-  // iff they there is a path from s1 to a final state that has the
-  // same label as some path from s2 to a final state.
-  CommonFuture *common_future = new CommonFuture(ifst);
-
-  DeterminizeFstOptions<Arc, Div, Filt> nopts;
+  using CommonDivisor = DefaultCommonDivisor<Weight>;
+  using Filter = RelationDeterminizeFilter<Arc, CommonFuture>;
+  // Subset elements with states s1 and s2 (resp.) are in this relation iff they
+  // there is a path from s1 to a final state that has the same label as some
+  // path from s2 to a final state.
+  auto *common_future = new CommonFuture(ifst);
+  DeterminizeFstOptions<Arc, CommonDivisor, Filter> nopts;
   nopts.delta = opts.delta;
   nopts.subsequential_label = opts.subsequential_label;
-  nopts.filter = new Filt(ifst, common_future, &head_);
+  nopts.filter = new Filter(ifst, common_future, &head_);
+  // The filter takes ownership of 'common_future', and determinization takes
+  // ownership of the filter itself.
   nopts.gc_limit = 0;  // Cache only the last state for fastest copy.
   if (opts.weight_threshold != Weight::Zero() ||
       opts.state_threshold != kNoStateId) {
@@ -409,16 +406,14 @@ void Disambiguator<Arc>::PreDisambiguate(const ExpandedFst<Arc> &ifst,
 template <class Arc>
 void Disambiguator<Arc>::FindAmbiguities(const ExpandedFst<Arc> &fst) {
   if (fst.Start() == kNoStateId) return;
-
-  candidates_ = new ArcIdMap(ArcIdCompare(head_));
-
-  std::pair<StateId, StateId> start_pr(fst.Start(), fst.Start());
+  candidates_.reset(new ArcIdMap(ArcIdCompare(head_)));
+  const auto start_pr = std::make_pair(fst.Start(), fst.Start());
   coreachable_.insert(start_pr);
   queue_.push_back(start_pr);
   while (!queue_.empty()) {
-    const std::pair<StateId, StateId> &pr = queue_.front();
-    StateId s1 = pr.first;
-    StateId s2 = pr.second;
+    const auto &pr = queue_.front();
+    const auto s1 = pr.first;
+    const auto s2 = pr.second;
     queue_.pop_front();
     FindAmbiguousPairs(fst, s1, s2);
   }
@@ -428,44 +423,31 @@ template <class Arc>
 void Disambiguator<Arc>::FindAmbiguousPairs(const ExpandedFst<Arc> &fst,
                                             StateId s1, StateId s2) {
   if (fst.NumArcs(s2) > fst.NumArcs(s1)) FindAmbiguousPairs(fst, s2, s1);
-
   SortedMatcher<Fst<Arc>> matcher(fst, MATCH_INPUT);
   matcher.SetState(s2);
   for (ArcIterator<Fst<Arc>> aiter(fst, s1); !aiter.Done(); aiter.Next()) {
-    const Arc &arc1 = aiter.Value();
-    ArcId a1(s1, aiter.Position());
+    const auto &arc1 = aiter.Value();
+    const ArcId a1(s1, aiter.Position());
     if (matcher.Find(arc1.ilabel)) {
       for (; !matcher.Done(); matcher.Next()) {
-        const Arc &arc2 = matcher.Value();
-        if (arc2.ilabel == kNoLabel)  // implicit epsilon match
-          continue;
-        ArcId a2(s2, matcher.Position());
-
-        // Actual transition is ambiguous
+        const auto &arc2 = matcher.Value();
+        // Continues on implicit epsilon match.
+        if (arc2.ilabel == kNoLabel) continue;
+        const ArcId a2(s2, matcher.Position());
+        // Actual transition is ambiguous.
         if (s1 != s2 && arc1.nextstate == arc2.nextstate) {
-          std::pair<ArcId, ArcId> apr;
-          if (head_[s1] > head_[s2]) {
-            apr = std::make_pair(a1, a2);
-          } else {
-            apr = std::make_pair(a2, a1);
-          }
-          candidates_->insert(apr);
-        }
-
-        std::pair<StateId, StateId> spr;
-        if (arc1.nextstate <= arc2.nextstate) {
-          spr = std::make_pair(arc1.nextstate, arc2.nextstate);
-        } else {
-          spr = std::make_pair(arc2.nextstate, arc1.nextstate);
+          InsertCandidate(s1, s2, a1, a2);
         }
+        const auto spr = arc1.nextstate <= arc2.nextstate
+                             ? std::make_pair(arc1.nextstate, arc2.nextstate)
+                             : std::make_pair(arc2.nextstate, arc1.nextstate);
         // Not already marked as coreachable?
-        if (coreachable_.find(spr) == coreachable_.end()) {
-          coreachable_.insert(spr);
-          // Only possible if state split by quantization issues
+        if (coreachable_.insert(spr).second) {
+          // Only possible if state split by quantization issues.
           if (spr.first != spr.second &&
               head_[spr.first] == head_[spr.second]) {
             if (!merge_) {
-              merge_ = new UnionFind<StateId>(fst.NumStates(), kNoStateId);
+              merge_.reset(new UnionFind<StateId>(fst.NumStates(), kNoStateId));
               merge_->MakeAllSet(fst.NumStates());
             }
             merge_->Union(spr.first, spr.second);
@@ -476,62 +458,50 @@ void Disambiguator<Arc>::FindAmbiguousPairs(const ExpandedFst<Arc> &fst,
       }
     }
   }
-
-  // Super-final transition is ambiguous
+  // Super-final transition is ambiguous.
   if (s1 != s2 && fst.Final(s1) != Weight::Zero() &&
       fst.Final(s2) != Weight::Zero()) {
-    ArcId a1(s1, -1);
-    ArcId a2(s2, -1);
-    std::pair<ArcId, ArcId> apr;
-    if (head_[s1] > head_[s2]) {
-      apr = std::make_pair(a1, a2);
-    } else {
-      apr = std::make_pair(a2, a1);
-    }
-    candidates_->insert(apr);
+    const ArcId a1(s1, -1);
+    const ArcId a2(s2, -1);
+    InsertCandidate(s1, s2, a1, a2);
   }
 }
 
 template <class Arc>
 void Disambiguator<Arc>::MarkAmbiguities() {
   if (!candidates_) return;
-
   for (auto it = candidates_->begin(); it != candidates_->end(); ++it) {
-    ArcId a = it->first;
-    ArcId b = it->second;
-    if (ambiguous_.count(b) == 0)  // if b is not to be removed
-      ambiguous_.insert(a);        // ... then a is.
+    const auto a = it->first;
+    const auto b = it->second;
+    // If b is not to be removed, then a is.
+    if (ambiguous_.count(b) == 0) ambiguous_.insert(a);
   }
   coreachable_.clear();
-  delete candidates_;
-  candidates_ = 0;
+  candidates_.reset();
 }
 
 template <class Arc>
 void Disambiguator<Arc>::RemoveSplits(MutableFst<Arc> *ofst) {
   if (!merge_) return;
-
-  // 'Merges' split states to remove spurious ambiguities
-  for (StateId s = 0; s < ofst->NumStates(); ++s) {
-    for (MutableArcIterator<MutableFst<Arc>> aiter(ofst, s); !aiter.Done();
-         aiter.Next()) {
-      Arc arc = aiter.Value();
-      StateId nextstate = merge_->FindSet(arc.nextstate);
+  // Merges split states to remove spurious ambiguities.
+  for (StateIterator<MutableFst<Arc>> siter(*ofst); !siter.Done();
+       siter.Next()) {
+    for (MutableArcIterator<MutableFst<Arc>> aiter(ofst, siter.Value());
+         !aiter.Done(); aiter.Next()) {
+      auto arc = aiter.Value();
+      const auto nextstate = merge_->FindSet(arc.nextstate);
       if (nextstate != arc.nextstate) {
         arc.nextstate = nextstate;
         aiter.SetValue(arc);
       }
     }
   }
-
-  // Repeats search for actual ambiguities on modified FST
+  // Repeats search for actual ambiguities on modified FST.
   coreachable_.clear();
-  delete merge_;
-  merge_ = 0;
-  delete candidates_;
-  candidates_ = 0;
+  merge_.reset();
+  candidates_.reset();
   FindAmbiguities(*ofst);
-  if (merge_) {  // shouldn't get here; sanity test
+  if (merge_) {  // Shouldn't get here; sanity test.
     FSTERROR() << "Disambiguate: Unable to remove spurious ambiguities";
     error_ = true;
     return;
@@ -541,51 +511,52 @@ void Disambiguator<Arc>::RemoveSplits(MutableFst<Arc> *ofst) {
 template <class Arc>
 void Disambiguator<Arc>::RemoveAmbiguities(MutableFst<Arc> *ofst) {
   if (ambiguous_.empty()) return;
-  // Adds dead state to redirect ambiguous transitions to be removed
-  StateId dead = ofst->AddState();
+  // Adds dead state to redirect ambiguous transitions to be removed.
+  const auto dead = ofst->AddState();
   for (auto it = ambiguous_.begin(); it != ambiguous_.end(); ++it) {
-    StateId s = it->first;
-    ssize_t pos = it->second;
-    if (pos >= 0) {  // actual transition
-      MutableArcIterator<MutableFst<Arc>> aiter(ofst, s);
+    const auto pos = it->second;
+    if (pos >= 0) {  // Actual transition.
+      MutableArcIterator<MutableFst<Arc>> aiter(ofst, it->first);
       aiter.Seek(pos);
-      Arc arc = aiter.Value();
+      auto arc = aiter.Value();
       arc.nextstate = dead;
       aiter.SetValue(arc);
-    } else {  // super-final transition
-      ofst->SetFinal(s, Weight::Zero());
+    } else {  // Super-final transition.
+      ofst->SetFinal(it->first, Weight::Zero());
     }
   }
   Connect(ofst);
   ambiguous_.clear();
 }
 
-// Disambiguates a weighted FST. This version writes the
-// disambiguated FST to an output MutableFst. The result will
-// be an equivalent FST that has the property that there are not
-// two distinct paths from the initial state to a final state
-// with the same input labeling.
+}  // namespace internal
+
+// Disambiguates a weighted FST. This version writes the disambiguated FST to an
+// output MutableFst. The result will be an equivalent FST that has the
+// property that there are not two distinct paths from the initial state to a
+// final state with the same input labeling.
 //
-// The weights must be (weakly) left divisible (valid for Tropical
-// and LogWeight).
+// The weights must be (weakly) left divisible (valid for Tropical and
+// LogWeight).
 //
 // Complexity:
-// - Disambiguable: exponential (polynomial in the size of the output)
-// - Non-disambiguable: does not terminate
 //
-// The disambiguable transducers include all automata and functional
-// transducers that are unweighted or that are acyclic or that are unambiguous.
+//   Disambiguable: exponential (polynomial in the size of the output).
+//   Non-disambiguable: does not terminate.
 //
-// References:
-// - Mehryar Mohri and Michael Riley, "On the Disambiguation
-//   of Weighted Automata," ArXiv e-prints, cs-FL/1405.0500, 2014,
-//   http://arxiv.org/abs/1405.0500.
+// The disambiguable transducers include all automata and functional transducers
+// that are unweighted or that are acyclic or that are unambiguous.
+//
+// For more information, see:
+//
+// Mohri, M. and Riley, M. 2015. On the disambiguation of weighted automata.
+// In CIAA, pages 263-278.
 template <class Arc>
 void Disambiguate(
     const Fst<Arc> &ifst, MutableFst<Arc> *ofst,
     const DisambiguateOptions<Arc> &opts = DisambiguateOptions<Arc>()) {
-  Disambiguator<Arc> disamb;
-  disamb.Disambiguate(ifst, ofst, opts);
+  internal::Disambiguator<Arc> disambiguator;
+  disambiguator.Disambiguate(ifst, ofst, opts);
 }
 
 }  // namespace fst
