@@ -3,8 +3,8 @@
 //
 // Extracts component FSTs from an finite-state archive.
 
-#ifndef FST_EXTENSIONS_FAR_EXTRACT_H__
-#define FST_EXTENSIONS_FAR_EXTRACT_H__
+#ifndef FST_EXTENSIONS_FAR_EXTRACT_H_
+#define FST_EXTENSIONS_FAR_EXTRACT_H_
 
 #include <string>
 #include <vector>
@@ -15,26 +15,25 @@ namespace fst {
 
 template <class Arc>
 inline void FarWriteFst(const Fst<Arc> *fst, string key, string *okey,
-                        int *nrep, const int32 &generate_filenames, int i,
+                        int *nrep, int32 generate_filenames, int i,
                         const string &filename_prefix,
                         const string &filename_suffix) {
-  if (key == *okey)
+  if (key == *okey) {
     ++*nrep;
-  else
+  } else {
     *nrep = 0;
-
+  }
   *okey = key;
-
   string ofilename;
   if (generate_filenames) {
-    ostringstream tmp;
+    std::ostringstream tmp;
     tmp.width(generate_filenames);
     tmp.fill('0');
     tmp << i;
     ofilename = tmp.str();
   } else {
     if (*nrep > 0) {
-      ostringstream tmp;
+      std::ostringstream tmp;
       tmp << '.' << nrep;
       key.append(tmp.str().data(), tmp.str().size());
     }
@@ -44,58 +43,56 @@ inline void FarWriteFst(const Fst<Arc> *fst, string key, string *okey,
 }
 
 template <class Arc>
-void FarExtract(const std::vector<string> &ifilenames,
-                const int32 &generate_filenames, const string &keys,
-                const string &key_separator, const string &range_delimiter,
-                const string &filename_prefix, const string &filename_suffix) {
+void FarExtract(const std::vector<string> &ifilenames, int32 generate_filenames,
+                const string &keys, const string &key_separator,
+                const string &range_delimiter, const string &filename_prefix,
+                const string &filename_suffix) {
   FarReader<Arc> *far_reader = FarReader<Arc>::Open(ifilenames);
   if (!far_reader) return;
-
   string okey;
   int nrep = 0;
-
   std::vector<char *> key_vector;
-  // User has specified a set of fsts to extract, where some of the "fsts" could
-  // be ranges.
+  // User has specified a set of FSTs to extract, where some of these may in
+  // fact be ranges.
   if (!keys.empty()) {
-    char *keys_cstr = new char[keys.size() + 1];
+    auto *keys_cstr = new char[keys.size() + 1];
     strcpy(keys_cstr, keys.c_str());
     SplitToVector(keys_cstr, key_separator.c_str(), &key_vector, true);
     int i = 0;
-    for (int k = 0; k < key_vector.size(); ++k, ++i) {
-      string key = string(key_vector[k]);
-      char *key_cstr = new char[key.size() + 1];
+    for (size_t k = 0; k < key_vector.size(); ++k, ++i) {
+      string key = key_vector[k];
+      auto *key_cstr = new char[key.size() + 1];
       strcpy(key_cstr, key.c_str());
       std::vector<char *> range_vector;
       SplitToVector(key_cstr, range_delimiter.c_str(), &range_vector, false);
       if (range_vector.size() == 1) {  // Not a range
         if (!far_reader->Find(key)) {
-          LOG(ERROR) << "FarExtract: Cannot find key: " << key;
+          LOG(ERROR) << "FarExtract: Cannot find key " << key;
           return;
         }
-        const Fst<Arc> *fst = far_reader->GetFst();
+        const auto *fst = far_reader->GetFst();
         FarWriteFst(fst, key, &okey, &nrep, generate_filenames, i,
                     filename_prefix, filename_suffix);
       } else if (range_vector.size() == 2) {  // A legal range
-        string begin_key = string(range_vector[0]);
-        string end_key = string(range_vector[1]);
+        string begin_key = range_vector[0];
+        string end_key = range_vector[1];
         if (begin_key.empty() || end_key.empty()) {
-          LOG(ERROR) << "FarExtract: Illegal range specification: " << key;
+          LOG(ERROR) << "FarExtract: Illegal range specification " << key;
           return;
         }
         if (!far_reader->Find(begin_key)) {
-          LOG(ERROR) << "FarExtract: Cannot find key: " << begin_key;
+          LOG(ERROR) << "FarExtract: Cannot find key " << begin_key;
           return;
         }
         for (; !far_reader->Done(); far_reader->Next(), ++i) {
-          string ikey = far_reader->GetKey();
+          const auto &ikey = far_reader->GetKey();
           if (end_key < ikey) break;
-          const Fst<Arc> *fst = far_reader->GetFst();
+          const auto *fst = far_reader->GetFst();
           FarWriteFst(fst, ikey, &okey, &nrep, generate_filenames, i,
                       filename_prefix, filename_suffix);
         }
       } else {
-        LOG(ERROR) << "FarExtract: Illegal range specification: " << key;
+        LOG(ERROR) << "FarExtract: Illegal range specification " << key;
         return;
       }
       delete[] key_cstr;
@@ -103,10 +100,10 @@ void FarExtract(const std::vector<string> &ifilenames,
     delete[] keys_cstr;
     return;
   }
-  // Nothing specified: extract everything.
-  for (int i = 1; !far_reader->Done(); far_reader->Next(), ++i) {
-    string key = far_reader->GetKey();
-    const Fst<Arc> *fst = far_reader->GetFst();
+  // Nothing specified, so just extracts everything.
+  for (size_t i = 1; !far_reader->Done(); far_reader->Next(), ++i) {
+    const auto &key = far_reader->GetKey();
+    const auto *fst = far_reader->GetFst();
     FarWriteFst(fst, key, &okey, &nrep, generate_filenames, i, filename_prefix,
                 filename_suffix);
   }
@@ -115,4 +112,4 @@ void FarExtract(const std::vector<string> &ifilenames,
 
 }  // namespace fst
 
-#endif  // FST_EXTENSIONS_FAR_EXTRACT_H__
+#endif  // FST_EXTENSIONS_FAR_EXTRACT_H_
