@@ -6,6 +6,7 @@
 #ifndef FST_EXTENSIONS_FAR_FAR_CLASS_H_
 #define FST_EXTENSIONS_FAR_FAR_CLASS_H_
 
+#include <memory>
 #include <string>
 #include <vector>
 
@@ -36,58 +37,54 @@ class FarReaderImplBase {
   virtual ~FarReaderImplBase() {}
 };
 
-
 // Templated implementation.
-template <class A>
+template <class Arc>
 class FarReaderClassImpl : public FarReaderImplBase {
  public:
-  using Arc = A;
+  explicit FarReaderClassImpl(const string &filename)
+      : impl_(FarReader<Arc>::Open(filename)) {}
 
-  explicit FarReaderClassImpl(const string &filename) :
-      impl_(FarReader<A>::Open(filename)) {}
+  explicit FarReaderClassImpl(const std::vector<string> &filenames)
+      : impl_(FarReader<Arc>::Open(filenames)) {}
 
-  explicit FarReaderClassImpl(const std::vector<string> &filenames) :
-      impl_(FarReader<A>::Open(filenames)) {}
+  const string &ArcType() const final { return Arc::Type(); }
 
-  const string &ArcType() const override { return Arc::Type(); }
+  bool Done() const final { return impl_->Done(); }
 
-  bool Done() const override { return impl_->Done(); }
+  bool Error() const final { return impl_->Error(); }
 
-  bool Error() const override { return impl_->Error(); }
+  bool Find(const string &key) final { return impl_->Find(key); }
 
-  bool Find(const string &key) override { return impl_->Find(key); }
-
-  const FstClass *GetFstClass() const override {
+  const FstClass *GetFstClass() const final {
     fstc_.reset(new FstClass(*impl_->GetFst()));
     return fstc_.get();
   }
 
-  const string &GetKey() const override { return impl_->GetKey(); }
+  const string &GetKey() const final { return impl_->GetKey(); }
 
-  void Next() override { return impl_->Next(); }
+  void Next() final { return impl_->Next(); }
 
-  void Reset() override { impl_->Reset(); }
+  void Reset() final { impl_->Reset(); }
 
-  FarType Type() const override { return impl_->Type(); }
+  FarType Type() const final { return impl_->Type(); }
 
-  const FarReader<A> *GetImpl() const { return impl_.get(); }
+  const FarReader<Arc> *GetImpl() const { return impl_.get(); }
 
-  FarReader<A> *GetImpl() { return impl_.get(); }
+  FarReader<Arc> *GetImpl() { return impl_.get(); }
 
  private:
-  std::unique_ptr<FarReader<A>> impl_;
+  std::unique_ptr<FarReader<Arc>> impl_;
   mutable std::unique_ptr<FstClass> fstc_;
 };
 
 
 class FarReaderClass;
 
-typedef args::WithReturnValue<FarReaderClass *, const string &>
-    OpenFarReaderClassArgs1;
-
-typedef args::WithReturnValue<FarReaderClass *, const std::vector<string> &>
-    OpenFarReaderClassArgs2;
+using OpenFarReaderClassArgs1 =
+    args::WithReturnValue<FarReaderClass *, const string &>;
 
+using OpenFarReaderClassArgs2 =
+    args::WithReturnValue<FarReaderClass *, const std::vector<string> &>;
 
 // Untemplated user-facing class holding a templated pimpl.
 class FarReaderClass {
@@ -112,26 +109,26 @@ class FarReaderClass {
 
   FarType Type() const { return impl_->Type(); }
 
-  template <class A>
-  const FarReader<A> *GetFarReader() const {
-    if (A::Type() != ArcType()) return nullptr;
-    const FarReaderClassImpl<A> *typed_impl =
-        static_cast<FarReaderClassImpl<A> *>(impl_.get());
+  template <class Arc>
+  const FarReader<Arc> *GetFarReader() const {
+    if (Arc::Type() != ArcType()) return nullptr;
+    const FarReaderClassImpl<Arc> *typed_impl =
+        static_cast<FarReaderClassImpl<Arc> *>(impl_.get());
     return typed_impl->GetImpl();
   }
 
-  template <class A>
-  FarReader<A> *GetFarReader() {
-    if (A::Type() != ArcType()) return nullptr;
-    FarReaderClassImpl<A> *typed_impl =
-        static_cast<FarReaderClassImpl<A> *>(impl_.get());
+  template <class Arc>
+  FarReader<Arc> *GetFarReader() {
+    if (Arc::Type() != ArcType()) return nullptr;
+    FarReaderClassImpl<Arc> *typed_impl =
+        static_cast<FarReaderClassImpl<Arc> *>(impl_.get());
     return typed_impl->GetImpl();
   }
 
-  template <class A>
+  template <class Arc>
   friend void OpenFarReaderClass(OpenFarReaderClassArgs1 *args);
 
-  template <class A>
+  template <class Arc>
   friend void OpenFarReaderClass(OpenFarReaderClassArgs2 *args);
 
   // Defined in the CC.
@@ -141,35 +138,32 @@ class FarReaderClass {
   static FarReaderClass *Open(const std::vector<string> &filenames);
 
  private:
-  template <class A>
-  explicit FarReaderClass(FarReaderClassImpl<A> *impl) : impl_(impl) {}
+  template <class Arc>
+  explicit FarReaderClass(FarReaderClassImpl<Arc> *impl) : impl_(impl) {}
 
   std::unique_ptr<FarReaderImplBase> impl_;
-
-  DISALLOW_COPY_AND_ASSIGN(FarReaderClass);
 };
 
 // These exist solely for registration purposes; users should call the
 // static method FarReaderClass::Open instead.
 
-template <class A>
+template <class Arc>
 void OpenFarReaderClass(OpenFarReaderClassArgs1 *args) {
-  args->retval = new FarReaderClass(new FarReaderClassImpl<A>(args->args));
+  args->retval = new FarReaderClass(new FarReaderClassImpl<Arc>(args->args));
 }
 
-template <class A>
+template <class Arc>
 void OpenFarReaderClass(OpenFarReaderClassArgs2 *args) {
-  args->retval = new FarReaderClass(new FarReaderClassImpl<A>(args->args));
+  args->retval = new FarReaderClass(new FarReaderClassImpl<Arc>(args->args));
 }
 
-
 // FarWriter API.
 
 // Virtual interface implemented by each concrete FarWriterImpl<A>.
 class FarWriterImplBase {
  public:
-  // Unlike the library, this returns a boolean to signal failure due to
-  // non-conformant arc types.
+  // Unlike the lower-level library, this returns a boolean to signal failure
+  // due to non-conformant arc types.
   virtual bool Add(const string &key, const FstClass &fst) = 0;
   virtual const string &ArcType() const = 0;
   virtual bool Error() const = 0;
@@ -179,47 +173,44 @@ class FarWriterImplBase {
 
 
 // Templated implementation.
-template <class A>
+template <class Arc>
 class FarWriterClassImpl : public FarWriterImplBase {
  public:
-  using Arc = A;
-
   explicit FarWriterClassImpl(const string &filename,
-                              FarType type = FAR_DEFAULT) :
-      impl_(FarWriter<A>::Create(filename, type)) {}
+                              FarType type = FAR_DEFAULT)
+      : impl_(FarWriter<Arc>::Create(filename, type)) {}
 
-  bool Add(const string &key, const FstClass &fst) override {
+  bool Add(const string &key, const FstClass &fst) final {
     if (ArcType() != fst.ArcType()) {
       FSTERROR() << "Cannot write FST with " << fst.ArcType() << " arcs to "
                  << "FAR with " << ArcType() << " arcs";
       return false;
     }
-    impl_->Add(key, *(fst.GetFst<A>()));
+    impl_->Add(key, *(fst.GetFst<Arc>()));
     return true;
   }
 
-  const string &ArcType() const override { return Arc::Type(); }
+  const string &ArcType() const final { return Arc::Type(); }
 
-  bool Error() const override { return impl_->Error(); }
+  bool Error() const final { return impl_->Error(); }
 
-  FarType Type() const override { return impl_->Type(); }
+  FarType Type() const final { return impl_->Type(); }
 
-  const FarWriter<A> *GetImpl() const { return impl_.get(); }
+  const FarWriter<Arc> *GetImpl() const { return impl_.get(); }
 
-  FarWriter<A> *GetImpl() { return impl_.get(); }
+  FarWriter<Arc> *GetImpl() { return impl_.get(); }
 
  private:
-  std::unique_ptr<FarWriter<A>> impl_;
+  std::unique_ptr<FarWriter<Arc>> impl_;
 };
 
 
 class FarWriterClass;
 
-typedef args::Package<const string &, FarType> CreateFarWriterClassInnerArgs;
-
-typedef args::WithReturnValue<FarWriterClass *, CreateFarWriterClassInnerArgs>
-    CreateFarWriterClassArgs;
+using CreateFarWriterClassInnerArgs = args::Package<const string &, FarType>;
 
+using CreateFarWriterClassArgs =
+    args::WithReturnValue<FarWriterClass *, CreateFarWriterClassInnerArgs>;
 
 // Untemplated user-facing class holding a templated pimpl.
 class FarWriterClass {
@@ -239,40 +230,38 @@ class FarWriterClass {
 
   FarType Type() const { return impl_->Type(); }
 
-  template <class A>
-  const FarWriter<A> *GetFarWriter() const {
-    if (A::Type() != ArcType()) return nullptr;
-    const FarWriterClassImpl<A> *typed_impl =
-        static_cast<FarWriterClassImpl<A> *>(impl_.get());
+  template <class Arc>
+  const FarWriter<Arc> *GetFarWriter() const {
+    if (Arc::Type() != ArcType()) return nullptr;
+    const FarWriterClassImpl<Arc> *typed_impl =
+        static_cast<FarWriterClassImpl<Arc> *>(impl_.get());
     return typed_impl->GetImpl();
   }
 
-  template <class A>
-  FarWriter<A> *GetFarWriter() {
-    if (A::Type() != ArcType()) return nullptr;
-    FarWriterClassImpl<A> *typed_impl =
-        static_cast<FarWriterClassImpl<A> *>(impl_.get());
+  template <class Arc>
+  FarWriter<Arc> *GetFarWriter() {
+    if (Arc::Type() != ArcType()) return nullptr;
+    FarWriterClassImpl<Arc> *typed_impl =
+        static_cast<FarWriterClassImpl<Arc> *>(impl_.get());
     return typed_impl->GetImpl();
   }
 
-  template <class A>
+  template <class Arc>
   friend void CreateFarWriterClass(CreateFarWriterClassArgs *args);
 
  private:
-  template <class A>
-  explicit FarWriterClass(FarWriterClassImpl<A> *impl) : impl_(impl) {}
+  template <class Arc>
+  explicit FarWriterClass(FarWriterClassImpl<Arc> *impl) : impl_(impl) {}
 
   std::unique_ptr<FarWriterImplBase> impl_;
-
-  DISALLOW_COPY_AND_ASSIGN(FarWriterClass);
 };
 
 // This exists solely for registration purposes; users should call the
 // static method FarWriterClass::Create instead.
-template <class A>
+template <class Arc>
 void CreateFarWriterClass(CreateFarWriterClassArgs *args) {
-  args->retval = new FarWriterClass(new FarWriterClassImpl<A>(args->args.arg1,
-                                                              args->args.arg2));
+  args->retval = new FarWriterClass(
+      new FarWriterClassImpl<Arc>(args->args.arg1, args->args.arg2));
 }
 
 }  // namespace script
