@@ -3,8 +3,8 @@
 //
 // Composes a PDT and an FST.
 
-#ifndef FST_EXTENSIONS_PDT_COMPOSE_H__
-#define FST_EXTENSIONS_PDT_COMPOSE_H__
+#ifndef FST_EXTENSIONS_PDT_COMPOSE_H_
+#define FST_EXTENSIONS_PDT_COMPOSE_H_
 
 #include <list>
 
@@ -13,11 +13,11 @@
 
 namespace fst {
 
-// Return paren arcs for Find(kNoLabel).
-const uint32 kParenList = 0x00000001;
+// Returns paren arcs for Find(kNoLabel).
+constexpr uint32 kParenList = 0x00000001;
 
-// Return a kNolabel loop for Find(paren).
-const uint32 kParenLoop = 0x00000002;
+// Returns a kNolabel loop for Find(paren).
+constexpr uint32 kParenLoop = 0x00000002;
 
 // This class is a matcher that treats parens as multi-epsilon labels.
 // It is most efficient if the parens are in a range non-overlapping with
@@ -25,12 +25,12 @@ const uint32 kParenLoop = 0x00000002;
 template <class F>
 class ParenMatcher {
  public:
-  typedef SortedMatcher<F> M;
-  typedef typename M::FST FST;
-  typedef typename M::Arc Arc;
-  typedef typename Arc::StateId StateId;
-  typedef typename Arc::Label Label;
-  typedef typename Arc::Weight Weight;
+  using FST = F;
+  using M = SortedMatcher<FST>;
+  using Arc = typename FST::Arc;
+  using Label = typename Arc::Label;
+  using StateId = typename Arc::StateId;
+  using Weight = typename Arc::Weight;
 
   ParenMatcher(const FST &fst, MatchType match_type,
                uint32 flags = (kParenLoop | kParenList))
@@ -46,7 +46,7 @@ class ParenMatcher {
     loop_.nextstate = kNoStateId;
   }
 
-  ParenMatcher(const ParenMatcher<F> &matcher, bool safe = false)
+  ParenMatcher(const ParenMatcher<FST> &matcher, bool safe = false)
       : matcher_(matcher.matcher_, safe),
         match_type_(matcher.match_type_),
         flags_(matcher.flags_),
@@ -56,8 +56,8 @@ class ParenMatcher {
     loop_.nextstate = kNoStateId;
   }
 
-  ParenMatcher<F> *Copy(bool safe = false) const {
-    return new ParenMatcher<F>(*this, safe);
+  ParenMatcher<FST> *Copy(bool safe = false) const {
+    return new ParenMatcher<FST>(*this, safe);
   }
 
   MatchType Type(bool test) const { return matcher_.Type(test); }
@@ -126,41 +126,35 @@ class ParenMatcher {
   bool IsCloseParen(Label label) const { return close_parens_.Member(label); }
 
  private:
-  // Advances matcher to next open paren if it exists, returning true.
-  // O.w. returns false.
+  // Advances matcher to next open paren, returning true if it exists.
   bool NextOpenParen();
 
-  // Advances matcher to next open paren if it exists, returning true.
-  // O.w. returns false.
+  // Advances matcher to next close paren, returning true if it exists.
   bool NextCloseParen();
 
   M matcher_;
-  MatchType match_type_;  // Type of match to perform
+  MatchType match_type_;  // Type of match to perform.
   uint32 flags_;
-
-  // open paren label set
+  // Open paren label set.
   CompactSet<Label, kNoLabel> open_parens_;
-
-  // close paren label set
+  // Close paren label set.
   CompactSet<Label, kNoLabel> close_parens_;
+  bool open_paren_list_;   // Matching open paren list?
+  bool close_paren_list_;  // Matching close paren list?
+  bool paren_loop_;        // Current arc is the implicit paren loop?
+  mutable Arc loop_;       // For non-consuming symbols.
+  bool done_;              // Matching done?
 
-  bool open_paren_list_;   // Matching open paren list
-  bool close_paren_list_;  // Matching close paren list
-  bool paren_loop_;        // Current arc is the implicit paren loop
-  mutable Arc loop_;       // For non-consuming symbols
-  bool done_;              // Matching done
-
-  void operator=(const ParenMatcher<F> &);  // Disallow
+  ParenMatcher &operator=(const ParenMatcher &) = delete;
 };
 
-template <class M>
-inline bool ParenMatcher<M>::Find(Label match_label) {
+template <class FST>
+inline bool ParenMatcher<FST>::Find(Label match_label) {
   open_paren_list_ = false;
   close_paren_list_ = false;
   paren_loop_ = false;
   done_ = false;
-
-  // Returns all parenthesis arcs
+  // Returns all parenthesis arcs.
   if (match_label == kNoLabel && (flags_ & kParenList)) {
     if (open_parens_.LowerBound() != kNoLabel) {
       matcher_.LowerBound(open_parens_.LowerBound());
@@ -173,23 +167,20 @@ inline bool ParenMatcher<M>::Find(Label match_label) {
       if (close_paren_list_) return true;
     }
   }
-
-  // Returns 'implicit' paren loop
+  // Returns the implicit paren loop.
   if (match_label > 0 && (flags_ & kParenLoop) &&
       (IsOpenParen(match_label) || IsCloseParen(match_label))) {
     paren_loop_ = true;
     return true;
   }
-
-  // Returns all other labels
+  // Returns all other labels.
   if (matcher_.Find(match_label)) return true;
-
   done_ = true;
   return false;
 }
 
-template <class F>
-inline void ParenMatcher<F>::Next() {
+template <class FST>
+inline void ParenMatcher<FST>::Next() {
   if (paren_loop_) {
     paren_loop_ = false;
     done_ = true;
@@ -197,7 +188,6 @@ inline void ParenMatcher<F>::Next() {
     matcher_.Next();
     open_paren_list_ = NextOpenParen();
     if (open_paren_list_) return;
-
     if (close_parens_.LowerBound() != kNoLabel) {
       matcher_.LowerBound(close_parens_.LowerBound());
       close_paren_list_ = NextCloseParen();
@@ -215,10 +205,9 @@ inline void ParenMatcher<F>::Next() {
   }
 }
 
-// Advances matcher to next open paren if it exists, returning true.
-// O.w. returns false.
-template <class F>
-inline bool ParenMatcher<F>::NextOpenParen() {
+// Advances matcher to next open paren, returning true if it exists.
+template <class FST>
+inline bool ParenMatcher<FST>::NextOpenParen() {
   for (; !matcher_.Done(); matcher_.Next()) {
     Label label = match_type_ == MATCH_INPUT ? matcher_.Value().ilabel
                                              : matcher_.Value().olabel;
@@ -228,10 +217,9 @@ inline bool ParenMatcher<F>::NextOpenParen() {
   return false;
 }
 
-// Advances matcher to next close paren if it exists, returning true.
-// O.w. returns false.
-template <class F>
-inline bool ParenMatcher<F>::NextCloseParen() {
+// Advances matcher to next close paren, returning true if it exists.
+template <class FST>
+inline bool ParenMatcher<FST>::NextCloseParen() {
   for (; !matcher_.Done(); matcher_.Next()) {
     Label label = match_type_ == MATCH_INPUT ? matcher_.Value().ilabel
                                              : matcher_.Value().olabel;
@@ -241,55 +229,55 @@ inline bool ParenMatcher<F>::NextCloseParen() {
   return false;
 }
 
-template <class F>
+template <class Filter>
 class ParenFilter {
  public:
-  typedef typename F::FST1 FST1;
-  typedef typename F::FST2 FST2;
-  typedef typename F::Arc Arc;
-  typedef typename Arc::StateId StateId;
-  typedef typename Arc::Label Label;
-  typedef typename Arc::Weight Weight;
-  typedef typename F::Matcher1 Matcher1;
-  typedef typename F::Matcher2 Matcher2;
-  typedef typename F::FilterState FilterState1;
-  typedef StateId StackId;
-  typedef PdtStack<StackId, Label> ParenStack;
-  typedef IntegerFilterState<StackId> FilterState2;
-  typedef PairFilterState<FilterState1, FilterState2> FilterState;
-  typedef ParenFilter<F> Filter;
+  using FST1 = typename Filter::FST1;
+  using FST2 = typename Filter::FST2;
+  using Arc = typename Filter::Arc;
+  using Label = typename Arc::Label;
+  using StateId = typename Arc::StateId;
+  using Weight = typename Arc::Weight;
 
-  ParenFilter(const FST1 &fst1, const FST2 &fst2, Matcher1 *matcher1 = 0,
-              Matcher2 *matcher2 = 0,
-              const std::vector<std::pair<Label, Label>> *parens = 0,
+  using Matcher1 = typename Filter::Matcher1;
+  using Matcher2 = typename Filter::Matcher2;
+
+  using StackId = StateId;
+  using ParenStack = PdtStack<StackId, Label>;
+  using FilterState1 = typename Filter::FilterState;
+  using FilterState2 = IntegerFilterState<StackId>;
+  using FilterState = PairFilterState<FilterState1, FilterState2>;
+
+  ParenFilter(const FST1 &fst1, const FST2 &fst2, Matcher1 *matcher1 = nullptr,
+              Matcher2 *matcher2 = nullptr,
+              const std::vector<std::pair<Label, Label>> *parens = nullptr,
               bool expand = false, bool keep_parens = true)
       : filter_(fst1, fst2, matcher1, matcher2),
         parens_(parens ? *parens : std::vector<std::pair<Label, Label>>()),
         expand_(expand),
         keep_parens_(keep_parens),
-        f_(FilterState::NoState()),
+        fs_(FilterState::NoState()),
         stack_(parens_),
         paren_id_(-1) {
     if (parens) {
-      for (size_t i = 0; i < parens->size(); ++i) {
-        const std::pair<Label, Label> &p = (*parens)[i];
-        parens_.push_back(p);
-        GetMatcher1()->AddOpenParen(p.first);
-        GetMatcher2()->AddOpenParen(p.first);
+      for (const auto &pair : *parens) {
+        parens_.push_back(pair);
+        GetMatcher1()->AddOpenParen(pair.first);
+        GetMatcher2()->AddOpenParen(pair.first);
         if (!expand_) {
-          GetMatcher1()->AddCloseParen(p.second);
-          GetMatcher2()->AddCloseParen(p.second);
+          GetMatcher1()->AddCloseParen(pair.second);
+          GetMatcher2()->AddCloseParen(pair.second);
         }
       }
     }
   }
 
-  ParenFilter(const Filter &filter, bool safe = false)
+  ParenFilter(const ParenFilter &filter, bool safe = false)
       : filter_(filter.filter_, safe),
         parens_(filter.parens_),
         expand_(filter.expand_),
         keep_parens_(filter.keep_parens_),
-        f_(FilterState::NoState()),
+        fs_(FilterState::NoState()),
         stack_(filter.parens_),
         paren_id_(-1) {}
 
@@ -297,12 +285,11 @@ class ParenFilter {
     return FilterState(filter_.Start(), FilterState2(0));
   }
 
-  void SetState(StateId s1, StateId s2, const FilterState &f) {
-    f_ = f;
-    filter_.SetState(s1, s2, f_.GetState1());
+  void SetState(StateId s1, StateId s2, const FilterState &fs) {
+    fs_ = fs;
+    filter_.SetState(s1, s2, fs_.GetState1());
     if (!expand_) return;
-
-    ssize_t paren_id = stack_.Top(f.GetState2().GetState());
+    ssize_t paren_id = stack_.Top(fs.GetState2().GetState());
     if (paren_id != paren_id_) {
       if (paren_id_ != -1) {
         GetMatcher1()->RemoveCloseParen(parens_[paren_id_].second);
@@ -317,80 +304,80 @@ class ParenFilter {
   }
 
   FilterState FilterArc(Arc *arc1, Arc *arc2) const {
-    FilterState1 f1 = filter_.FilterArc(arc1, arc2);
-    const FilterState2 &f2 = f_.GetState2();
-    if (f1 == FilterState1::NoState()) return FilterState::NoState();
-
-    if (arc1->olabel == kNoLabel && arc2->ilabel) {  // arc2 parentheses
+    const auto fs1 = filter_.FilterArc(arc1, arc2);
+    const auto &fs2 = fs_.GetState2();
+    if (fs1 == FilterState1::NoState()) return FilterState::NoState();
+    if (arc1->olabel == kNoLabel && arc2->ilabel) {  // arc2 parentheses.
       if (keep_parens_) {
         arc1->ilabel = arc2->ilabel;
       } else if (arc2->ilabel) {
         arc2->olabel = arc1->ilabel;
       }
-      return FilterParen(arc2->ilabel, f1, f2);
-    } else if (arc2->ilabel == kNoLabel && arc1->olabel) {  // arc1 parentheses
+      return FilterParen(arc2->ilabel, fs1, fs2);
+    } else if (arc2->ilabel == kNoLabel && arc1->olabel) {  // arc1 parentheses.
       if (keep_parens_) {
         arc2->olabel = arc1->olabel;
       } else {
         arc1->ilabel = arc2->olabel;
       }
-      return FilterParen(arc1->olabel, f1, f2);
+      return FilterParen(arc1->olabel, fs1, fs2);
     } else {
-      return FilterState(f1, f2);
+      return FilterState(fs1, fs2);
     }
   }
 
   void FilterFinal(Weight *w1, Weight *w2) const {
-    if (f_.GetState2().GetState() != 0) *w1 = Weight::Zero();
+    if (fs_.GetState2().GetState() != 0) *w1 = Weight::Zero();
     filter_.FilterFinal(w1, w2);
   }
 
-  // Return resp matchers. Ownership stays with filter.
+  // Returns respective matchers; ownership stays with filter.
+
   Matcher1 *GetMatcher1() { return filter_.GetMatcher1(); }
+
   Matcher2 *GetMatcher2() { return filter_.GetMatcher2(); }
 
   uint64 Properties(uint64 iprops) const {
-    uint64 oprops = filter_.Properties(iprops);
-    return oprops & kILabelInvariantProperties & kOLabelInvariantProperties;
+    return filter_.Properties(iprops) & kILabelInvariantProperties &
+           kOLabelInvariantProperties;
   }
 
  private:
-  const FilterState FilterParen(Label label, const FilterState1 &f1,
-                                const FilterState2 &f2) const {
-    if (!expand_) return FilterState(f1, f2);
-
-    StackId stack_id = stack_.Find(f2.GetState(), label);
+  const FilterState FilterParen(Label label, const FilterState1 &fs1,
+                                const FilterState2 &fs2) const {
+    if (!expand_) return FilterState(fs1, fs2);
+    const auto stack_id = stack_.Find(fs2.GetState(), label);
     if (stack_id < 0) {
       return FilterState::NoState();
     } else {
-      return FilterState(f1, FilterState2(stack_id));
+      return FilterState(fs1, FilterState2(stack_id));
     }
   }
 
-  F filter_;
+  Filter filter_;
   std::vector<std::pair<Label, Label>> parens_;
-  bool expand_;       // Expands to FST
-  bool keep_parens_;  // Retains parentheses in output
-  FilterState f_;     // Current filter state
+  bool expand_;       // Expands to FST?
+  bool keep_parens_;  // Retains parentheses in output?
+  FilterState fs_;    // Current filter state.
   mutable ParenStack stack_;
   ssize_t paren_id_;
 };
 
-// Class to setup composition options for PDT composition.
-// Default is for the PDT as the first composition argument.
+// Class to setup composition options for PDT composition. Default is to take
+// the PDT as the first composition argument.
 template <class Arc, bool left_pdt = true>
 class PdtComposeFstOptions
     : public ComposeFstOptions<
           Arc, ParenMatcher<Fst<Arc>>,
-          ParenFilter<AltSequenceComposeFilter<ParenMatcher<Fst<Arc>> >> > {
+          ParenFilter<AltSequenceComposeFilter<ParenMatcher<Fst<Arc>>>>> {
  public:
-  typedef typename Arc::Label Label;
-  typedef ParenMatcher<Fst<Arc>> PdtMatcher;
-  typedef ParenFilter<AltSequenceComposeFilter<PdtMatcher>> PdtFilter;
-  typedef ComposeFstOptions<Arc, PdtMatcher, PdtFilter> COptions;
-  using COptions::matcher1;
-  using COptions::matcher2;
-  using COptions::filter;
+  using Label = typename Arc::Label;
+  using PdtMatcher = ParenMatcher<Fst<Arc>>;
+  using PdtFilter = ParenFilter<AltSequenceComposeFilter<PdtMatcher>>;
+
+  using ComposeFstOptions<Arc, PdtMatcher, PdtFilter>::matcher1;
+  using ComposeFstOptions<Arc, PdtMatcher, PdtFilter>::matcher2;
+  using ComposeFstOptions<Arc, PdtMatcher, PdtFilter>::filter;
 
   PdtComposeFstOptions(const Fst<Arc> &ifst1,
                        const std::vector<std::pair<Label, Label>> &parens,
@@ -398,7 +385,6 @@ class PdtComposeFstOptions
                        bool keep_parens = true) {
     matcher1 = new PdtMatcher(ifst1, MATCH_OUTPUT, kParenList);
     matcher2 = new PdtMatcher(ifst2, MATCH_INPUT, kParenLoop);
-
     filter = new PdtFilter(ifst1, ifst2, matcher1, matcher2, &parens, expand,
                            keep_parens);
   }
@@ -410,48 +396,46 @@ template <class Arc>
 class PdtComposeFstOptions<Arc, false>
     : public ComposeFstOptions<
           Arc, ParenMatcher<Fst<Arc>>,
-          ParenFilter<SequenceComposeFilter<ParenMatcher<Fst<Arc>> >> > {
+          ParenFilter<SequenceComposeFilter<ParenMatcher<Fst<Arc>>>>> {
  public:
-  typedef typename Arc::Label Label;
-  typedef ParenMatcher<Fst<Arc>> PdtMatcher;
-  typedef ParenFilter<SequenceComposeFilter<PdtMatcher>> PdtFilter;
-  typedef ComposeFstOptions<Arc, PdtMatcher, PdtFilter> COptions;
-  using COptions::matcher1;
-  using COptions::matcher2;
-  using COptions::filter;
+  using Label = typename Arc::Label;
+  using PdtMatcher = ParenMatcher<Fst<Arc>>;
+  using PdtFilter = ParenFilter<SequenceComposeFilter<PdtMatcher>>;
+
+  using ComposeFstOptions<Arc, PdtMatcher, PdtFilter>::matcher1;
+  using ComposeFstOptions<Arc, PdtMatcher, PdtFilter>::matcher2;
+  using ComposeFstOptions<Arc, PdtMatcher, PdtFilter>::filter;
 
   PdtComposeFstOptions(const Fst<Arc> &ifst1, const Fst<Arc> &ifst2,
                        const std::vector<std::pair<Label, Label>> &parens,
                        bool expand = false, bool keep_parens = true) {
     matcher1 = new PdtMatcher(ifst1, MATCH_OUTPUT, kParenLoop);
     matcher2 = new PdtMatcher(ifst2, MATCH_INPUT, kParenList);
-
     filter = new PdtFilter(ifst1, ifst2, matcher1, matcher2, &parens, expand,
                            keep_parens);
   }
 };
 
 enum PdtComposeFilter {
-  PAREN_FILTER,         // Bar-Hillel construction; keeps parentheses
-  EXPAND_FILTER,        // Bar-Hillel + expansion; removes parentheses
-  EXPAND_PAREN_FILTER,  // Bar-Hillel + expansion; keeps parentheses
+  PAREN_FILTER,         // Bar-Hillel construction; keeps parentheses.
+  EXPAND_FILTER,        // Bar-Hillel + expansion; removes parentheses.
+  EXPAND_PAREN_FILTER,  // Bar-Hillel + expansion; keeps parentheses.
 };
 
 struct PdtComposeOptions {
-  bool connect;                  // Connect output
-  PdtComposeFilter filter_type;  // Which pre-defined filter to use
+  bool connect;                  // Connect output?
+  PdtComposeFilter filter_type;  // Pre-defined filter to use.
 
-  explicit PdtComposeOptions(bool c, PdtComposeFilter ft = PAREN_FILTER)
-      : connect(c), filter_type(ft) {}
-  PdtComposeOptions() : connect(true), filter_type(PAREN_FILTER) {}
+  explicit PdtComposeOptions(bool connect = true,
+                             PdtComposeFilter filter_type = PAREN_FILTER)
+      : connect(connect), filter_type(filter_type) {}
 };
 
-// Composes pushdown transducer (PDT) encoded as an FST (1st arg) and
-// an FST (2nd arg) with the result also a PDT encoded as an Fst. (3rd arg).
-// In the PDTs, some transitions are labeled with open or close
-// parentheses. To be interpreted as a PDT, the parens must balance on
-// a path (see PdtExpand()). The open-close parenthesis label pairs
-// are passed in 'parens'.
+// Composes pushdown transducer (PDT) encoded as an FST (1st arg) and an FST
+// (2nd arg) with the result also a PDT encoded as an FST (3rd arg). In the
+// PDTs, some transitions are labeled with open or close parentheses. To be
+// interpreted as a PDT, the parens must balance on a path (see PdtExpand()).
+// The open-close parenthesis label pairs are passed using the parens argument.
 template <class Arc>
 void Compose(const Fst<Arc> &ifst1,
              const std::vector<
@@ -467,12 +451,11 @@ void Compose(const Fst<Arc> &ifst1,
   if (opts.connect) Connect(ofst);
 }
 
-// Composes an FST (1st arg) and pushdown transducer (PDT) encoded as
-// an FST (2nd arg) with the result also a PDT encoded as an Fst (3rd arg).
-// In the PDTs, some transitions are labeled with open or close
-// parentheses. To be interpreted as a PDT, the parens must balance on
-// a path (see ExpandFst()). The open-close parenthesis label pairs
-// are passed in 'parens'.
+// Composes an FST (1st arg) and pushdown transducer (PDT) encoded as an FST
+// (2nd arg) with the result also a PDT encoded as an FST (3rd arg). In the
+// PDTs, some transitions are labeled with open or close parentheses. To be
+// interpreted as a PDT, the parens must balance on a path (see ExpandFst()).
+// The open-close parenthesis label pairs are passed using the parens argument.
 template <class Arc>
 void Compose(const Fst<Arc> &ifst1, const Fst<Arc> &ifst2,
              const std::vector<
@@ -490,4 +473,4 @@ void Compose(const Fst<Arc> &ifst1, const Fst<Arc> &ifst2,
 
 }  // namespace fst
 
-#endif  // FST_EXTENSIONS_PDT_COMPOSE_H__
+#endif  // FST_EXTENSIONS_PDT_COMPOSE_H_
