@@ -7,6 +7,7 @@
 #define FST_LIB_FILTER_STATE_H_
 
 #include <forward_list>
+#include <utility>
 
 #include <fst/fst-decl.h>  // For optional argument declarations
 #include <fst/fst.h>
@@ -15,24 +16,31 @@
 
 namespace fst {
 
-// FILTER STATE - this represents the state of the algorithm
-// (e.g. composition) filter. It has the form:
+// The filter state interface represents the state of a (e.g., composition)
+// filter.
 //
 // class FilterState {
 //  public:
-//   // Required constructors
+//   // Required constructors.
+//
 //   FilterState();
-//   FilterState(const FilterState &f);
+//
+//   FilterState(const FilterState &fs);
+//
 //   // An invalid filter state.
 //   static const FilterState NoState();
+//
 //   // Maps state to integer for hashing.
 //   size_t Hash() const;
+//
 //   // Equality of filter states.
-//   bool operator==(const FilterState &f) const;
+//   bool operator==(const FilterState &fs) const;
+//
 //   // Inequality of filter states.
-//   bool operator!=(const FilterState &f) const;
+//   bool operator!=(const FilterState &fs) const;
+//
 //   // Assignment to filter states.
-//   FilterState& operator=(const FilterState& f);
+//   FilterState &operator=(const FilterState& fs);
 // };
 
 // Filter state that is a signed integral type.
@@ -40,18 +48,19 @@ template <typename T>
 class IntegerFilterState {
  public:
   IntegerFilterState() : state_(kNoStateId) {}
+
   explicit IntegerFilterState(T s) : state_(s) {}
 
   static const IntegerFilterState NoState() { return IntegerFilterState(); }
 
   size_t Hash() const { return static_cast<size_t>(state_); }
 
-  bool operator==(const IntegerFilterState &f) const {
-    return state_ == f.state_;
+  bool operator==(const IntegerFilterState &fs) const {
+    return state_ == fs.state_;
   }
 
-  bool operator!=(const IntegerFilterState &f) const {
-    return state_ != f.state_;
+  bool operator!=(const IntegerFilterState &fs) const {
+    return state_ != fs.state_;
   }
 
   T GetState() const { return state_; }
@@ -62,32 +71,33 @@ class IntegerFilterState {
   T state_;
 };
 
-typedef IntegerFilterState<signed char> CharFilterState;
-typedef IntegerFilterState<short> ShortFilterState;
-typedef IntegerFilterState<int> IntFilterState;
+using CharFilterState = IntegerFilterState<signed char>;
+using ShortFilterState = IntegerFilterState<short>;  // NOLINT
+using IntFilterState = IntegerFilterState<int>;
 
 // Filter state that is a weight (class).
 template <class W>
 class WeightFilterState {
  public:
   WeightFilterState() : weight_(W::Zero()) {}
-  explicit WeightFilterState(W w) : weight_(w) {}
+
+  explicit WeightFilterState(W weight) : weight_(std::move(weight)) {}
 
   static const WeightFilterState NoState() { return WeightFilterState(); }
 
   size_t Hash() const { return weight_.Hash(); }
 
-  bool operator==(const WeightFilterState &f) const {
-    return weight_ == f.weight_;
+  bool operator==(const WeightFilterState &fs) const {
+    return weight_ == fs.weight_;
   }
 
-  bool operator!=(const WeightFilterState &f) const {
-    return weight_ != f.weight_;
+  bool operator!=(const WeightFilterState &fs) const {
+    return weight_ != fs.weight_;
   }
 
   W GetWeight() const { return weight_; }
 
-  void SetWeight(W w) { weight_ = w; }
+  void SetWeight(W weight) { weight_ = std::move(weight); }
 
  private:
   W weight_;
@@ -106,16 +116,13 @@ class ListFilterState {
 
   size_t Hash() const {
     size_t h = 0;
-    for (typename std::forward_list<T>::const_iterator iter = list_.begin();
-         iter != list_.end(); ++iter) {
-      h ^= h << 1 ^ *iter;
-    }
+    for (const auto &elem : list_) h ^= h << 1 ^ elem;
     return h;
   }
 
-  bool operator==(const ListFilterState &f) const { return list_ == f.list_; }
+  bool operator==(const ListFilterState &fs) const { return list_ == fs.list_; }
 
-  bool operator!=(const ListFilterState &f) const { return list_ != f.list_; }
+  bool operator!=(const ListFilterState &fs) const { return list_ != fs.list_; }
 
   const std::forward_list<T> &GetState() const { return list_; }
 
@@ -128,59 +135,60 @@ class ListFilterState {
 };
 
 // Filter state that is the combination of two filter states.
-template <class F1, class F2>
+template <class FS1, class FS2>
 class PairFilterState {
  public:
-  PairFilterState() : f1_(F1::NoState()), f2_(F2::NoState()) {}
+  PairFilterState() : fs1_(FS1::NoState()), fs2_(FS2::NoState()) {}
 
-  PairFilterState(const F1 &f1, const F2 &f2) : f1_(f1), f2_(f2) {}
+  PairFilterState(const FS1 &fs1, const FS2 &fs2) : fs1_(fs1), fs2_(fs2) {}
 
   static const PairFilterState NoState() { return PairFilterState(); }
 
   size_t Hash() const {
-    size_t h1 = f1_.Hash();
-    size_t h2 = f2_.Hash();
-    const int lshift = 5;
-    const int rshift = CHAR_BIT * sizeof(size_t) - 5;
-    return h1 << lshift ^ h1 >> rshift ^ h2;
+    const auto h1 = fs1_.Hash();
+    static constexpr auto lshift = 5;
+    static constexpr auto rshift = CHAR_BIT * sizeof(size_t) - 5;
+    return h1 << lshift ^ h1 >> rshift ^ fs2_.Hash();
   }
 
-  bool operator==(const PairFilterState &f) const {
-    return f1_ == f.f1_ && f2_ == f.f2_;
+  bool operator==(const PairFilterState &fs) const {
+    return fs1_ == fs.fs1_ && fs2_ == fs.fs2_;
   }
 
-  bool operator!=(const PairFilterState &f) const {
-    return f1_ != f.f1_ || f2_ != f.f2_;
+  bool operator!=(const PairFilterState &fs) const {
+    return fs1_ != fs.fs1_ || fs2_ != fs.fs2_;
   }
 
-  const F1 &GetState1() const { return f1_; }
-  const F2 &GetState2() const { return f2_; }
+  const FS1 &GetState1() const { return fs1_; }
 
-  void SetState(const F1 &f1, const F2 &f2) {
-    f1_ = f1;
-    f2_ = f2;
+  const FS2 &GetState2() const { return fs2_; }
+
+  void SetState(const FS1 &fs1, const FS2 &fs2) {
+    fs1_ = fs1;
+    fs2_ = fs2;
   }
 
  private:
-  F1 f1_;
-  F2 f2_;
+  FS1 fs1_;
+  FS2 fs2_;
 };
 
 // Single non-blocking filter state.
 class TrivialFilterState {
  public:
-  TrivialFilterState() : state_(false) {}
-  explicit TrivialFilterState(bool s) : state_(s) {}
+  explicit TrivialFilterState(bool state = false) : state_(state) {}
 
   static const TrivialFilterState NoState() { return TrivialFilterState(); }
 
   size_t Hash() const { return 0; }
 
-  bool operator==(const TrivialFilterState &f) const {
-    return state_ == f.state_; }
+  bool operator==(const TrivialFilterState &fs) const {
+    return state_ == fs.state_;
+  }
 
-  bool operator!=(const TrivialFilterState &f) const {
-    return state_ != f.state_; }
+  bool operator!=(const TrivialFilterState &fs) const {
+    return state_ != fs.state_;
+  }
 
  private:
   bool state_;
