@@ -11,12 +11,11 @@
 namespace fst {
 namespace script {
 
-typedef args::Package<MutableFstClass *, const ClosureType> ClosureArgs;
+using ClosureArgs = args::Package<MutableFstClass *, const ClosureType>;
 
 template <class Arc>
 void Closure(ClosureArgs *args) {
   MutableFst<Arc> *fst = args->arg1->GetMutableFst<Arc>();
-
   Closure(fst, args->arg2);
 }
 
