@@ -11,14 +11,14 @@
 namespace fst {
 namespace script {
 
-typedef args::Package<const FstClass &, const FstClass &, float> EqualInnerArgs;
-typedef args::WithReturnValue<bool, EqualInnerArgs> EqualArgs;
+using EqualInnerArgs = args::Package<const FstClass &, const FstClass &, float>;
+
+using EqualArgs = args::WithReturnValue<bool, EqualInnerArgs>;
 
 template <class Arc>
 void Equal(EqualArgs *args) {
   const Fst<Arc> &fst1 = *(args->args.arg1.GetFst<Arc>());
   const Fst<Arc> &fst2 = *(args->args.arg2.GetFst<Arc>());
-
   args->retval = Equal(fst1, fst2, args->args.arg3);
 }
 
