@@ -38,8 +38,9 @@ struct ShortestDistanceOptions {
 };
 
 // 1
-typedef args::Package<const FstClass &, std::vector<WeightClass> *,
-                      const ShortestDistanceOptions &> ShortestDistanceArgs1;
+using ShortestDistanceArgs1 =
+    args::Package<const FstClass &, std::vector<WeightClass> *,
+                  const ShortestDistanceOptions &>;
 
 template <class Queue, class Arc, class ArcFilter>
 struct QueueConstructor {
@@ -89,47 +90,43 @@ void ShortestDistanceHelper(ShortestDistanceArgs1 *args) {
   std::vector<typename Arc::Weight> weights;
   switch (opts.arc_filter_type) {
     case ANY_ARC_FILTER: {
-      Queue *queue = QueueConstructor<Queue, Arc, AnyArcFilter<Arc>>::Construct(
-          fst, &weights);
+      std::unique_ptr<Queue> queue(QueueConstructor<Queue, Arc,
+          AnyArcFilter<Arc>>::Construct(fst, &weights));
       fst::ShortestDistanceOptions<Arc, Queue, AnyArcFilter<Arc>> sdopts(
-          queue, AnyArcFilter<Arc>(), opts.source, opts.delta);
+          queue.get(), AnyArcFilter<Arc>(), opts.source, opts.delta);
       ShortestDistance(fst, &weights, sdopts);
-      delete queue;
       break;
     }
     case EPSILON_ARC_FILTER: {
-      Queue *queue = QueueConstructor<Queue, Arc, AnyArcFilter<Arc>>::Construct(
-          fst, &weights);
+      std::unique_ptr<Queue> queue(QueueConstructor<Queue, Arc,
+          AnyArcFilter<Arc>>::Construct(fst, &weights));
       fst::ShortestDistanceOptions<Arc, Queue, EpsilonArcFilter<Arc>>
-          sdopts(queue, EpsilonArcFilter<Arc>(), opts.source, opts.delta);
+          sdopts(queue.get(), EpsilonArcFilter<Arc>(), opts.source, opts.delta);
       ShortestDistance(fst, &weights, sdopts);
-      delete queue;
       break;
     }
     case INPUT_EPSILON_ARC_FILTER: {
-      Queue *queue =
-          QueueConstructor<Queue, Arc, InputEpsilonArcFilter<Arc>>::Construct(
-              fst, &weights);
+      std::unique_ptr<Queue> queue(QueueConstructor<Queue, Arc,
+          InputEpsilonArcFilter<Arc>>::Construct(fst, &weights));
       fst::ShortestDistanceOptions<Arc, Queue, InputEpsilonArcFilter<Arc>>
-          sdopts(queue, InputEpsilonArcFilter<Arc>(), opts.source, opts.delta);
+          sdopts(queue.get(), InputEpsilonArcFilter<Arc>(), opts.source,
+                 opts.delta);
       ShortestDistance(fst, &weights, sdopts);
-      delete queue;
       break;
     }
     case OUTPUT_EPSILON_ARC_FILTER: {
-      Queue *queue =
-          QueueConstructor<Queue, Arc, OutputEpsilonArcFilter<Arc>>::Construct(
-              fst, &weights);
+      std::unique_ptr<Queue> queue(QueueConstructor<Queue, Arc,
+          OutputEpsilonArcFilter<Arc>>::Construct(fst, &weights));
       fst::ShortestDistanceOptions<Arc, Queue, OutputEpsilonArcFilter<Arc>>
-          sdopts(queue, OutputEpsilonArcFilter<Arc>(), opts.source, opts.delta);
+          sdopts(queue.get(), OutputEpsilonArcFilter<Arc>(), opts.source,
+                 opts.delta);
       ShortestDistance(fst, &weights, sdopts);
-      delete queue;
       break;
     }
   }
   // Copies the weights back.
   args->arg2->resize(weights.size());
-  for (unsigned i = 0; i < weights.size(); ++i) {
+  for (auto i = 0; i < weights.size(); ++i) {
     (*args->arg2)[i] = WeightClass(weights[i]);
   }
 }
@@ -137,10 +134,9 @@ void ShortestDistanceHelper(ShortestDistanceArgs1 *args) {
 template <class Arc>
 void ShortestDistance(ShortestDistanceArgs1 *args) {
   const ShortestDistanceOptions &opts = args->arg3;
-  typedef typename Arc::StateId StateId;
-  typedef typename Arc::Weight Weight;
-
-  // Must consider (opts.queue_type x opts.filter_type) options
+  using StateId = typename Arc::StateId;
+  using Weight = typename Arc::Weight;
+  // Must consider (opts.queue_type x opts.filter_type) options.
   switch (opts.queue_type) {
     default:
       FSTERROR() << "Unknown queue type: " << opts.queue_type;
@@ -167,8 +163,8 @@ void ShortestDistance(ShortestDistanceArgs1 *args) {
 }
 
 // 2
-typedef args::Package<const FstClass &, std::vector<WeightClass> *, bool,
-                      double> ShortestDistanceArgs2;
+using ShortestDistanceArgs2 =
+    args::Package<const FstClass &, std::vector<WeightClass> *, bool, double>;
 
 template <class Arc>
 void ShortestDistance(ShortestDistanceArgs2 *args) {
@@ -182,16 +178,6 @@ void ShortestDistance(ShortestDistanceArgs2 *args) {
     (*retval)[i] = WeightClass(distance[i]);
 }
 
-// 3
-typedef args::WithReturnValue<WeightClass &, const FstClass &>
-    ShortestDistanceArgs3;
-
-template <class Arc>
-void ShortestDistance(ShortestDistanceArgs3 *args) {
-  const Fst<Arc> &fst = *(args->args.GetFst<Arc>());
-  args->retval = WeightClass(ShortestDistance(fst));
-}
-
 // 1
 void ShortestDistance(const FstClass &fst, std::vector<WeightClass> *distance,
                       const ShortestDistanceOptions &opts);
