@@ -9,6 +9,8 @@
 #include <deque>
 #include <vector>
 
+#include <fst/log.h>
+
 #include <fst/arcfilter.h>
 #include <fst/cache.h>
 #include <fst/queue.h>
@@ -20,11 +22,11 @@ namespace fst {
 
 template <class Arc, class Queue, class ArcFilter>
 struct ShortestDistanceOptions {
-  typedef typename Arc::StateId StateId;
+  using StateId = typename Arc::StateId;
 
-  Queue *state_queue;    // Queue discipline used; owned by caller
-  ArcFilter arc_filter;  // Arc filter (e.g., limit to only epsilon graph)
-  StateId source;        // If kNoStateId, use the Fst's initial state
+  Queue *state_queue;    // Queue discipline used; owned by caller.
+  ArcFilter arc_filter;  // Arc filter (e.g., limit to only epsilon graph).
+  StateId source;        // If kNoStateId, use the FST's initial state.
   float delta;           // Determines the degree of convergence required
   bool first_path;       // For a semiring with the path property (o.w.
                          // undefined), compute the shortest-distances along
@@ -36,28 +38,29 @@ struct ShortestDistanceOptions {
                          // weights in the FST are between One() and Zero()
                          // according to NaturalLess.
 
-  ShortestDistanceOptions(Queue *q, ArcFilter filt, StateId src = kNoStateId,
-                          float d = kDelta)
-      : state_queue(q),
-        arc_filter(filt),
-        source(src),
-        delta(d),
+  ShortestDistanceOptions(Queue *state_queue, ArcFilter arc_filter,
+                          StateId source = kNoStateId, float delta = kDelta)
+      : state_queue(state_queue),
+        arc_filter(arc_filter),
+        source(source),
+        delta(delta),
         first_path(false) {}
 };
 
-// Computation state of the shortest-distance algorithm. Reusable
-// information is maintained across calls to member function
-// ShortestDistance(source) when 'retain' is true for improved
-// efficiency when calling multiple times from different source states
-// (e.g., in epsilon removal). Contrary to usual conventions, 'fst'
-// may not be freed before this class. Vector 'distance' should not be
-// modified by the user between these calls.
-// The Error() method returns true if an error was encountered.
+namespace internal {
+
+// Computation state of the shortest-distance algorithm. Reusable information
+// is maintained across calls to member function ShortestDistance(source) when
+// retain is true for improved efficiency when calling multiple times from
+// different source states (e.g., in epsilon removal). Contrary to the usual
+// conventions, fst may not be freed before this class. Vector distance
+// should not be modified by the user between these calls. The Error() method
+// returns true iff an error was encountered.
 template <class Arc, class Queue, class ArcFilter>
 class ShortestDistanceState {
  public:
-  typedef typename Arc::StateId StateId;
-  typedef typename Arc::Weight Weight;
+  using StateId = typename Arc::StateId;
+  using Weight = typename Arc::Weight;
 
   ShortestDistanceState(
       const Fst<Arc> &fst, std::vector<Weight> *distance,
@@ -74,8 +77,6 @@ class ShortestDistanceState {
     distance_->clear();
   }
 
-  ~ShortestDistanceState() {}
-
   void ShortestDistance(StateId source);
 
   bool Error() const { return error_; }
@@ -85,21 +86,21 @@ class ShortestDistanceState {
   std::vector<Weight> *distance_;
   Queue *state_queue_;
   ArcFilter arc_filter_;
-  float delta_;
-  bool first_path_;
-  bool retain_;  // Retain and reuse information across calls
+  const float delta_;
+  const bool first_path_;
+  const bool retain_;  // Retain and reuse information across calls.
 
-  std::vector<Weight> rdistance_;  // Relaxation distance.
+  std::vector<Adder<Weight>> adder_;   // Sums distance_ accurately.
+  std::vector<Adder<Weight>> radder_;  // Relaxation distance.
   std::vector<bool> enqueued_;     // Is state enqueued?
-  std::vector<StateId> sources_;   // Source ID for ith state in 'distance_',
-  //  'rdistance_', and 'enqueued_' if retained.
-  StateId source_id_;         // Unique ID characterizing each call to SD
-
+  std::vector<StateId> sources_;   // Source ID for ith state in distance_,
+                                   // (r)adder_, and enqueued_ if retained.
+  StateId source_id_;              // Unique ID characterizing each call.
   bool error_;
 };
 
-// Compute the shortest distance. If 'source' is kNoStateId, use
-// the initial state of the Fst.
+// Compute the shortest distance; if source is kNoStateId, uses the initial
+// state of the FST.
 template <class Arc, class Queue, class ArcFilter>
 void ShortestDistanceState<Arc, Queue, ArcFilter>::ShortestDistance(
     StateId source) {
@@ -107,34 +108,30 @@ void ShortestDistanceState<Arc, Queue, ArcFilter>::ShortestDistance(
     if (fst_.Properties(kError, false)) error_ = true;
     return;
   }
-
   if (!(Weight::Properties() & kRightSemiring)) {
     FSTERROR() << "ShortestDistance: Weight needs to be right distributive: "
                << Weight::Type();
     error_ = true;
     return;
   }
-
   if (first_path_ && !(Weight::Properties() & kPath)) {
     FSTERROR() << "ShortestDistance: The first_path option is disallowed when "
                << "Weight does not have the path property: " << Weight::Type();
     error_ = true;
     return;
   }
-
   state_queue_->Clear();
-
   if (!retain_) {
     distance_->clear();
-    rdistance_.clear();
+    adder_.clear();
+    radder_.clear();
     enqueued_.clear();
   }
-
   if (source == kNoStateId) source = fst_.Start();
-
   while (distance_->size() <= source) {
     distance_->push_back(Weight::Zero());
-    rdistance_.push_back(Weight::Zero());
+    adder_.push_back(Adder<Weight>());
+    radder_.push_back(Adder<Weight>());
     enqueued_.push_back(false);
   }
   if (retain_) {
@@ -142,47 +139,51 @@ void ShortestDistanceState<Arc, Queue, ArcFilter>::ShortestDistance(
     sources_[source] = source_id_;
   }
   (*distance_)[source] = Weight::One();
-  rdistance_[source] = Weight::One();
+  adder_[source].Reset(Weight::One());
+  radder_[source].Reset(Weight::One());
   enqueued_[source] = true;
-
   state_queue_->Enqueue(source);
-
   while (!state_queue_->Empty()) {
-    StateId s = state_queue_->Head();
+    const auto state = state_queue_->Head();
     state_queue_->Dequeue();
-    while (distance_->size() <= s) {
+    while (distance_->size() <= state) {
       distance_->push_back(Weight::Zero());
-      rdistance_.push_back(Weight::Zero());
+      adder_.push_back(Adder<Weight>());
+      radder_.push_back(Adder<Weight>());
       enqueued_.push_back(false);
     }
-    if (first_path_ && (fst_.Final(s) != Weight::Zero())) break;
-    enqueued_[s] = false;
-    Weight r = rdistance_[s];
-    rdistance_[s] = Weight::Zero();
-    for (ArcIterator<Fst<Arc>> aiter(fst_, s); !aiter.Done(); aiter.Next()) {
-      const Arc &arc = aiter.Value();
+    if (first_path_ && (fst_.Final(state) != Weight::Zero())) break;
+    enqueued_[state] = false;
+    const auto r = radder_[state].Sum();
+    radder_[state].Reset();
+    for (ArcIterator<Fst<Arc>> aiter(fst_, state); !aiter.Done();
+         aiter.Next()) {
+      const auto &arc = aiter.Value();
       if (!arc_filter_(arc)) continue;
       while (distance_->size() <= arc.nextstate) {
         distance_->push_back(Weight::Zero());
-        rdistance_.push_back(Weight::Zero());
+        adder_.push_back(Adder<Weight>());
+        radder_.push_back(Adder<Weight>());
         enqueued_.push_back(false);
       }
       if (retain_) {
         while (sources_.size() <= arc.nextstate) sources_.push_back(kNoStateId);
         if (sources_[arc.nextstate] != source_id_) {
           (*distance_)[arc.nextstate] = Weight::Zero();
-          rdistance_[arc.nextstate] = Weight::Zero();
+          adder_[arc.nextstate].Reset();
+          radder_[arc.nextstate].Reset();
           enqueued_[arc.nextstate] = false;
           sources_[arc.nextstate] = source_id_;
         }
       }
-      Weight &nd = (*distance_)[arc.nextstate];
-      Weight &nr = rdistance_[arc.nextstate];
-      Weight w = Times(r, arc.weight);
-      if (!ApproxEqual(nd, Plus(nd, w), delta_)) {
-        nd = Plus(nd, w);
-        nr = Plus(nr, w);
-        if (!nd.Member() || !nr.Member()) {
+      auto &nd = (*distance_)[arc.nextstate];
+      auto &na = adder_[arc.nextstate];
+      auto &nr = radder_[arc.nextstate];
+      auto weight = Times(r, arc.weight);
+      if (!ApproxEqual(nd, Plus(nd, weight), delta_)) {
+        nd = na.Add(weight);
+        nr.Add(weight);
+        if (!nd.Member() || !nr.Sum().Member()) {
           error_ = true;
           return;
         }
@@ -199,32 +200,37 @@ void ShortestDistanceState<Arc, Queue, ArcFilter>::ShortestDistance(
   if (fst_.Properties(kError, false)) error_ = true;
 }
 
+}  // namespace internal
+
 // Shortest-distance algorithm: this version allows fine control
 // via the options argument. See below for a simpler interface.
 //
-// This computes the shortest distance from the 'opts.source' state to
-// each visited state S and stores the value in the 'distance' vector.
-// An unvisited state S has distance Zero(), which will be stored in
-// the 'distance' vector if S is less than the maximum visited state.
-// The state queue discipline, arc filter, and convergence delta are
-// taken in the options argument.
-// The 'distance' vector will contain a unique element for which
-// Member() is false if an error was encountered.
+// This computes the shortest distance from the opts.source state to each
+// visited state S and stores the value in the distance vector. An
+// nvisited state S has distance Zero(), which will be stored in the
+// distance vector if S is less than the maximum visited state. The state
+// queue discipline, arc filter, and convergence delta are taken in the
+// options argument. The distance vector will contain a unique element for
+// which Member() is false if an error was encountered.
 //
 // The weights must must be right distributive and k-closed (i.e., 1 +
 // x + x^2 + ... + x^(k +1) = 1 + x + x^2 + ... + x^k).
 //
-// The algorithm is from Mohri, "Semiring Framework and Algorithms for
-// Shortest-Distance Problems", Journal of Automata, Languages and
-// Combinatorics 7(3):321-350, 2002. The complexity of algorithm
-// depends on the properties of the semiring and the queue discipline
-// used. Refer to the paper for more details.
+// Complexity:
+//
+// Depends on properties of the semiring and the queue discipline.
+//
+// For more information, see:
+//
+// Mohri, M. 2002. Semiring framework and algorithms for shortest-distance
+// problems, Journal of Automata, Languages and
+// Combinatorics 7(3): 321-350, 2002.
 template <class Arc, class Queue, class ArcFilter>
 void ShortestDistance(
     const Fst<Arc> &fst, std::vector<typename Arc::Weight> *distance,
     const ShortestDistanceOptions<Arc, Queue, ArcFilter> &opts) {
-  ShortestDistanceState<Arc, Queue, ArcFilter> sd_state(fst, distance, opts,
-                                                        false);
+  internal::ShortestDistanceState<Arc, Queue, ArcFilter> sd_state(fst, distance,
+                                                                  opts, false);
   sd_state.ShortestDistance(opts.source);
   if (sd_state.Error()) {
     distance->clear();
@@ -232,93 +238,96 @@ void ShortestDistance(
   }
 }
 
-// Shortest-distance algorithm: simplified interface. See above for a
-// version that allows finer control.
+// Shortest-distance algorithm: simplified interface. See above for a version
+// that permits finer control.
 //
-// If 'reverse' is false, this computes the shortest distance from the
-// initial state to each state S and stores the value in the
-// 'distance' vector. If 'reverse' is true, this computes the shortest
-// distance from each state to the final states.  An unvisited state S
-// has distance Zero(), which will be stored in the 'distance' vector
-// if S is less than the maximum visited state.  The state queue
-// discipline is automatically-selected.
-// The 'distance' vector will contain a unique element for which
-// Member() is false if an error was encountered.
+// If reverse is false, this computes the shortest distance from the initial
+// state to each state S and stores the value in the distance vector. If
+// reverse is true, this computes the shortest distance from each state to the
+// final states. An unvisited state S has distance Zero(), which will be stored
+// in the distance vector if S is less than the maximum visited state. The
+// state queue discipline is automatically-selected. The distance vector will
+// contain a unique element for which Member() is false if an error was
+// encountered.
 //
-// The weights must must be right (left) distributive if reverse is
-// false (true) and k-closed (i.e., 1 + x + x^2 + ... + x^(k +1) = 1 +
-// x + x^2 + ... + x^k).
+// The weights must must be right (left) distributive if reverse is false (true)
+// and k-closed (i.e., 1 + x + x^2 + ... + x^(k +1) = 1 + x + x^2 + ... + x^k).
 //
 // Arc weights must satisfy the property that the sum of the weights of one or
 // more paths from some state S to T is never Zero(). In particular, arc weights
 // are never Zero().
 //
-// The algorithm is from Mohri, "Semiring Framweork and Algorithms for
-// Shortest-Distance Problems", Journal of Automata, Languages and
-// Combinatorics 7(3):321-350, 2002. The complexity of algorithm
-// depends on the properties of the semiring and the queue discipline
-// used. Refer to the paper for more details.
+// Complexity:
+//
+// Depends on properties of the semiring and the queue discipline.
+//
+// For more information, see:
+//
+// Mohri, M. 2002. Semiring framework and algorithms for
+// shortest-distance problems, Journal of Automata, Languages and
+// Combinatorics 7(3): 321-350, 2002.
 template <class Arc>
 void ShortestDistance(const Fst<Arc> &fst,
                       std::vector<typename Arc::Weight> *distance,
                       bool reverse = false, float delta = kDelta) {
-  typedef typename Arc::StateId StateId;
-  typedef typename Arc::Weight Weight;
-
+  using StateId = typename Arc::StateId;
+  using Weight = typename Arc::Weight;
   if (!reverse) {
     AnyArcFilter<Arc> arc_filter;
     AutoQueue<StateId> state_queue(fst, distance, arc_filter);
-    ShortestDistanceOptions<Arc, AutoQueue<StateId>, AnyArcFilter<Arc>> opts(
-        &state_queue, arc_filter);
-    opts.delta = delta;
+    const ShortestDistanceOptions<Arc, AutoQueue<StateId>, AnyArcFilter<Arc>>
+        opts(&state_queue, arc_filter, kNoStateId, delta);
     ShortestDistance(fst, distance, opts);
   } else {
-    typedef ReverseArc<Arc> ReverseArc;
-    typedef typename ReverseArc::Weight ReverseWeight;
+    using ReverseArc = ReverseArc<Arc>;
+    using ReverseWeight = typename ReverseArc::Weight;
     AnyArcFilter<ReverseArc> rarc_filter;
     VectorFst<ReverseArc> rfst;
     Reverse(fst, &rfst);
     std::vector<ReverseWeight> rdistance;
     AutoQueue<StateId> state_queue(rfst, &rdistance, rarc_filter);
-    ShortestDistanceOptions<ReverseArc, AutoQueue<StateId>,
-                            AnyArcFilter<ReverseArc>> ropts(&state_queue,
-                                                             rarc_filter);
-    ropts.delta = delta;
+    const ShortestDistanceOptions<ReverseArc, AutoQueue<StateId>,
+                                  AnyArcFilter<ReverseArc>>
+        ropts(&state_queue, rarc_filter, kNoStateId, delta);
     ShortestDistance(rfst, &rdistance, ropts);
     distance->clear();
     if (rdistance.size() == 1 && !rdistance[0].Member()) {
       distance->resize(1, Arc::Weight::NoWeight());
       return;
     }
-    while (distance->size() < rdistance.size() - 1)
+    while (distance->size() < rdistance.size() - 1) {
       distance->push_back(rdistance[distance->size() + 1].Reverse());
+    }
   }
 }
 
-// Return the sum of the weight of all successful paths in an FST, i.e.,
-// the shortest-distance from the initial state to the final states.
-// Returns a weight such that Member() is false if an error was encountered.
+// Return the sum of the weight of all successful paths in an FST, i.e., the
+// shortest-distance from the initial state to the final states. Returns a
+// weight such that Member() is false if an error was encountered.
 template <class Arc>
 typename Arc::Weight ShortestDistance(const Fst<Arc> &fst,
                                       float delta = kDelta) {
-  typedef typename Arc::Weight Weight;
-  typedef typename Arc::StateId StateId;
+  using StateId = typename Arc::StateId;
+  using Weight = typename Arc::Weight;
   std::vector<Weight> distance;
   if (Weight::Properties() & kRightSemiring) {
     ShortestDistance(fst, &distance, false, delta);
-    if (distance.size() == 1 && !distance[0].Member())
+    if (distance.size() == 1 && !distance[0].Member()) {
       return Arc::Weight::NoWeight();
-    Weight sum = Weight::Zero();
-    for (StateId s = 0; s < distance.size(); ++s)
-      sum = Plus(sum, Times(distance[s], fst.Final(s)));
-    return sum;
+    }
+    Adder<Weight> adder;  // maintains cumulative sum accurately
+    for (StateId state = 0; state < distance.size(); ++state) {
+      adder.Add(Times(distance[state], fst.Final(state)));
+    }
+    return adder.Sum();
   } else {
     ShortestDistance(fst, &distance, true, delta);
-    StateId s = fst.Start();
-    if (distance.size() == 1 && !distance[0].Member())
+    const auto state = fst.Start();
+    if (distance.size() == 1 && !distance[0].Member()) {
       return Arc::Weight::NoWeight();
-    return s != kNoStateId && s < distance.size() ? distance[s]
-                                                  : Weight::Zero();
+    }
+    return state != kNoStateId && state < distance.size() ? distance[state]
+                                                          : Weight::Zero();
   }
 }
 
