@@ -15,7 +15,7 @@ void PrintFst(const FstClass &fst, std::ostream &ostrm, const string &dest,
               const SymbolTable *isyms, const SymbolTable *osyms,
               const SymbolTable *ssyms, bool accept, bool show_weight_one,
               const string &missing_sym) {
-  string sep = FLAGS_fst_field_separator.substr(0, 1);
+  const auto sep = FLAGS_fst_field_separator.substr(0, 1);
   FstPrinterArgs args(fst, isyms, osyms, ssyms, accept, show_weight_one, &ostrm,
                       dest, sep, missing_sym);
   Apply<Operation<FstPrinterArgs>>("PrintFst", fst.ArcType(), &args);
