@@ -10,6 +10,7 @@
 #include <utility>
 
 #include <fst/types.h>
+#include <fst/log.h>
 #include <fstream>
 #include <fst/util.h>
 
@@ -18,59 +19,58 @@ namespace script {
 
 // Reads vector of weights; returns true on success.
 bool ReadPotentials(const string &weight_type, const string &filename,
-                    std::vector<WeightClass> *potential) {
-  std::ifstream strm(filename.c_str());
-  if (!strm.good()) {
+                    std::vector<WeightClass> *potentials) {
+  std::ifstream istrm(filename);
+  if (!istrm.good()) {
     LOG(ERROR) << "ReadPotentials: Can't open file: " << filename;
     return false;
   }
-
-  const int kLineLen = 8096;
+  static const int kLineLen = 8096;
   char line[kLineLen];
   size_t nline = 0;
-
-  potential->clear();
-  while (!strm.getline(line, kLineLen).fail()) {
+  potentials->clear();
+  while (!istrm.getline(line, kLineLen).fail()) {
     ++nline;
     std::vector<char *> col;
     SplitToVector(line, "\n\t ", &col, true);
-    if (col.size() == 0 || col[0][0] == '\0')  // empty line
-      continue;
+    if (col.size() == 0 || col[0][0] == '\0') continue;
     if (col.size() != 2) {
       FSTERROR() << "ReadPotentials: Bad number of columns, "
                  << "file = " << filename << ", line = " << nline;
       return false;
     }
-
-    ssize_t s = StrToInt64(col[0], filename, nline, false);
-    WeightClass weight(weight_type, col[1]);
-
-    while (potential->size() <= s)
-      potential->push_back(WeightClass::Zero(weight_type));
-    (*potential)[s] = weight;
+    const ssize_t s = StrToInt64(col[0], filename, nline, false);
+    const WeightClass weight(weight_type, col[1]);
+    while (potentials->size() <= s) {
+      potentials->push_back(WeightClass::Zero(weight_type));
+    }
+    potentials->back() = weight;
   }
   return true;
 }
 
 // Writes vector of weights; returns true on success.
 bool WritePotentials(const string &filename,
-                     const std::vector<WeightClass> &potential) {
-  std::ofstream fstrm;
+                     const std::vector<WeightClass> &potentials) {
+  std::ofstream ostrm;
   if (!filename.empty()) {
-    fstrm.open(filename.c_str());
-    if (!fstrm) {
+    ostrm.open(filename);
+    if (!ostrm.good()) {
       LOG(ERROR) << "WritePotentials: Can't open file: " << filename;
       return false;
     }
   }
-  std::ostream &ostrm = fstrm.is_open() ? fstrm : std::cout;
-  ostrm.precision(9);
-  for (size_t s = 0; s < potential.size(); ++s)
-    ostrm << s << "\t" << potential[s] << "\n";
-  if (ostrm.fail())
+  std::ostream &strm = ostrm.is_open() ? ostrm : std::cout;
+  strm.precision(9);
+  for (size_t s = 0; s < potentials.size(); ++s) {
+    strm << s << "\t" << potentials[s] << "\n";
+  }
+  if (strm.fail()) {
     LOG(ERROR) << "WritePotentials: Write failed: "
                << (filename.empty() ? "standard output" : filename);
-  return !ostrm;
+    return false;
+  }
+  return true;
 }
 
 }  // namespace script
